"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .layoutcontrolsdto import LayoutControlsDto, LayoutControlsDtoTypedDict
from .resourceoriginenum import ResourceOriginEnum
from .resourcetypeenum import ResourceTypeEnum
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdatedByTypedDict(TypedDict):
    r"""User who last updated the layout"""

    id: str
    r"""User ID"""
    first_name: NotRequired[Nullable[str]]
    r"""User first name"""
    last_name: NotRequired[Nullable[str]]
    r"""User last name"""
    external_id: NotRequired[Nullable[str]]
    r"""User external ID"""


class UpdatedBy(BaseModel):
    r"""User who last updated the layout"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""User ID"""

    first_name: Annotated[OptionalNullable[str], pydantic.Field(alias="firstName")] = (
        UNSET
    )
    r"""User first name"""

    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""User last name"""

    external_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="externalId")
    ] = UNSET
    r"""User external ID"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["firstName", "lastName", "externalId"]
        nullable_fields = ["firstName", "lastName", "externalId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class LayoutResponseDtoTypedDict(TypedDict):
    id: str
    r"""Unique internal identifier of the layout"""
    layout_id: str
    r"""Unique identifier for the layout"""
    slug: str
    r"""Slug of the layout"""
    name: str
    r"""Name of the layout"""
    is_default: bool
    r"""Whether the layout is the default layout"""
    is_translation_enabled: bool
    r"""Whether the layout translations are enabled"""
    updated_at: str
    r"""Last updated timestamp"""
    created_at: str
    r"""Creation timestamp"""
    origin: ResourceOriginEnum
    r"""Origin of the layout"""
    type: ResourceTypeEnum
    r"""Type of the layout"""
    controls: LayoutControlsDtoTypedDict
    r"""Controls metadata for the layout"""
    updated_by: NotRequired[Nullable[UpdatedByTypedDict]]
    r"""User who last updated the layout"""
    variables: NotRequired[Nullable[Dict[str, Any]]]
    r"""The variables JSON Schema for the layout"""


class LayoutResponseDto(BaseModel):
    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Unique internal identifier of the layout"""

    layout_id: Annotated[str, pydantic.Field(alias="layoutId")]
    r"""Unique identifier for the layout"""

    slug: str
    r"""Slug of the layout"""

    name: str
    r"""Name of the layout"""

    is_default: Annotated[bool, pydantic.Field(alias="isDefault")]
    r"""Whether the layout is the default layout"""

    is_translation_enabled: Annotated[
        bool, pydantic.Field(alias="isTranslationEnabled")
    ]
    r"""Whether the layout translations are enabled"""

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]
    r"""Last updated timestamp"""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""Creation timestamp"""

    origin: ResourceOriginEnum
    r"""Origin of the layout"""

    type: ResourceTypeEnum
    r"""Type of the layout"""

    controls: LayoutControlsDto
    r"""Controls metadata for the layout"""

    updated_by: Annotated[
        OptionalNullable[UpdatedBy], pydantic.Field(alias="updatedBy")
    ] = UNSET
    r"""User who last updated the layout"""

    variables: OptionalNullable[Dict[str, Any]] = UNSET
    r"""The variables JSON Schema for the layout"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["updatedBy", "variables"]
        nullable_fields = ["updatedBy", "variables"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class UpdateTopicRequestDtoTypedDict(TypedDict):
    name: str
    r"""The display name for the topic"""


class UpdateTopicRequestDto(BaseModel):
    name: str
    r"""The display name for the topic"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubscriberFeedResponseDtoTypedDict(TypedDict):
    subscriber_id: str
    r"""The identifier used to create this subscriber, which typically corresponds to the user ID in your system."""
    id: NotRequired[str]
    r"""The internal ID generated by Novu for your subscriber. This ID does not match the `subscriberId` used in your queries. Refer to `subscriberId` for that identifier."""
    first_name: NotRequired[str]
    r"""The first name of the subscriber."""
    last_name: NotRequired[str]
    r"""The last name of the subscriber."""
    avatar: NotRequired[str]
    r"""The URL of the subscriber's avatar image."""


class SubscriberFeedResponseDto(BaseModel):
    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""The identifier used to create this subscriber, which typically corresponds to the user ID in your system."""

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""The internal ID generated by Novu for your subscriber. This ID does not match the `subscriberId` used in your queries. Refer to `subscriberId` for that identifier."""

    first_name: Annotated[Optional[str], pydantic.Field(alias="firstName")] = None
    r"""The first name of the subscriber."""

    last_name: Annotated[Optional[str], pydantic.Field(alias="lastName")] = None
    r"""The last name of the subscriber."""

    avatar: Optional[str] = None
    r"""The URL of the subscriber's avatar image."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class MsTeamsUserEndpointDtoTypedDict(TypedDict):
    user_id: str
    r"""MS Teams user ID"""


class MsTeamsUserEndpointDto(BaseModel):
    user_id: Annotated[str, pydantic.Field(alias="userId")]
    r"""MS Teams user ID"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .topicdto import TopicDto, TopicDtoTypedDict
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubscriptionDtoSubscriberTypedDict(TypedDict):
    r"""The subscriber information"""

    id: str
    r"""The identifier of the subscriber"""
    subscriber_id: str
    r"""The external identifier of the subscriber"""
    avatar: NotRequired[Nullable[str]]
    r"""The avatar URL of the subscriber"""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the subscriber"""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the subscriber"""
    email: NotRequired[Nullable[str]]
    r"""The email of the subscriber"""


class SubscriptionDtoSubscriber(BaseModel):
    r"""The subscriber information"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""The identifier of the subscriber"""

    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""The external identifier of the subscriber"""

    avatar: OptionalNullable[str] = UNSET
    r"""The avatar URL of the subscriber"""

    first_name: Annotated[OptionalNullable[str], pydantic.Field(alias="firstName")] = (
        UNSET
    )
    r"""The first name of the subscriber"""

    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""The last name of the subscriber"""

    email: OptionalNullable[str] = UNSET
    r"""The email of the subscriber"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["avatar", "firstName", "lastName", "email"]
        nullable_fields = ["avatar", "firstName", "lastName", "email"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class SubscriptionDtoTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the subscription"""
    topic: TopicDtoTypedDict
    r"""The topic information"""
    subscriber: Nullable[SubscriptionDtoSubscriberTypedDict]
    r"""The subscriber information"""
    created_at: str
    r"""The creation date of the subscription"""
    updated_at: str
    r"""The last update date of the subscription"""
    identifier: NotRequired[str]
    r"""The identifier of the subscription"""


class SubscriptionDto(BaseModel):
    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""The unique identifier of the subscription"""

    topic: TopicDto
    r"""The topic information"""

    subscriber: Nullable[SubscriptionDtoSubscriber]
    r"""The subscriber information"""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The creation date of the subscription"""

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]
    r"""The last update date of the subscription"""

    identifier: Optional[str] = None
    r"""The identifier of the subscription"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["identifier"]
        nullable_fields = ["subscriber"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .textalignenum import TextAlignEnum
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class EmailBlockStylesTypedDict(TypedDict):
    text_align: TextAlignEnum
    r"""Text alignment for the email block"""


class EmailBlockStyles(BaseModel):
    text_align: Annotated[TextAlignEnum, pydantic.Field(alias="textAlign")]
    r"""Text alignment for the email block"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
from novu_py.utils import FieldMetadata
import pydantic
from typing_extensions import Annotated, TypedDict


class UploadTranslationsRequestDtoResourceType(str, Enum):
    r"""The resource type to associate localizations with"""

    WORKFLOW = "workflow"
    LAYOUT = "layout"


class UploadTranslationsRequestDtoTypedDict(TypedDict):
    resource_id: str
    r"""The resource ID to associate localizations with. Accepts identifier or slug format"""
    resource_type: UploadTranslationsRequestDtoResourceType
    r"""The resource type to associate localizations with"""


class UploadTranslationsRequestDto(BaseModel):
    resource_id: Annotated[
        str, pydantic.Field(alias="resourceId"), FieldMetadata(multipart=True)
    ]
    r"""The resource ID to associate localizations with. Accepts identifier or slug format"""

    resource_type: Annotated[
        UploadTranslationsRequestDtoResourceType,
        pydantic.Field(alias="resourceType"),
        FieldMetadata(multipart=True),
    ]
    r"""The resource type to associate localizations with"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .actiondto import ActionDto, ActionDtoTypedDict
from .redirectdto import RedirectDto, RedirectDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class InAppControlDtoTypedDict(TypedDict):
    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    body: NotRequired[str]
    r"""Content/body of the in-app message. Required if subject is empty."""
    subject: NotRequired[str]
    r"""Subject/title of the in-app message. Required if body is empty."""
    avatar: NotRequired[str]
    r"""URL for an avatar image. Must be a valid URL or start with / or {{ variable }}."""
    primary_action: NotRequired[ActionDtoTypedDict]
    r"""Primary action button details."""
    secondary_action: NotRequired[ActionDtoTypedDict]
    r"""Secondary action button details."""
    redirect: NotRequired[RedirectDtoTypedDict]
    r"""Redirection URL configuration for the main content click (if no actions defined/clicked).."""
    disable_output_sanitization: NotRequired[bool]
    r"""Disable sanitization of the output."""
    data: NotRequired[Dict[str, Any]]
    r"""Additional data payload for the step."""


class InAppControlDto(BaseModel):
    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    body: Optional[str] = None
    r"""Content/body of the in-app message. Required if subject is empty."""

    subject: Optional[str] = None
    r"""Subject/title of the in-app message. Required if body is empty."""

    avatar: Optional[str] = None
    r"""URL for an avatar image. Must be a valid URL or start with / or {{ variable }}."""

    primary_action: Annotated[
        Optional[ActionDto], pydantic.Field(alias="primaryAction")
    ] = None
    r"""Primary action button details."""

    secondary_action: Annotated[
        Optional[ActionDto], pydantic.Field(alias="secondaryAction")
    ] = None
    r"""Secondary action button details."""

    redirect: Optional[RedirectDto] = None
    r"""Redirection URL configuration for the main content click (if no actions defined/clicked).."""

    disable_output_sanitization: Annotated[
        Optional[bool], pydantic.Field(alias="disableOutputSanitization")
    ] = False
    r"""Disable sanitization of the output."""

    data: Optional[Dict[str, Any]] = None
    r"""Additional data payload for the step."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class RemoveSubscriberResponseDtoTypedDict(TypedDict):
    acknowledged: bool
    r"""Indicates whether the operation was acknowledged by the server"""
    status: str
    r"""Status of the subscriber removal operation"""


class RemoveSubscriberResponseDto(BaseModel):
    acknowledged: bool
    r"""Indicates whether the operation was acknowledged by the server"""

    status: str
    r"""Status of the subscriber removal operation"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class GenerateChatOAuthURLResponseDtoTypedDict(TypedDict):
    url: str
    r"""The OAuth authorization URL for the chat provider. For Slack: https://slack.com/oauth/v2/authorize?... For MS Teams: https://login.microsoftonline.com/.../adminconsent?... This URL should be presented to the user to authorize the integration. Expires after 5 minutes."""


class GenerateChatOAuthURLResponseDto(BaseModel):
    url: str
    r"""The OAuth authorization URL for the chat provider. For Slack: https://slack.com/oauth/v2/authorize?... For MS Teams: https://login.microsoftonline.com/.../adminconsent?... This URL should be presented to the user to authorize the integration. Expires after 5 minutes."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PatchWorkflowDtoTypedDict(TypedDict):
    active: NotRequired[bool]
    r"""Activate or deactivate the workflow"""
    name: NotRequired[str]
    r"""New name for the workflow"""
    description: NotRequired[str]
    r"""Updated description of the workflow"""
    tags: NotRequired[List[str]]
    r"""Tags associated with the workflow"""
    payload_schema: NotRequired[Nullable[Dict[str, Any]]]
    r"""The payload JSON Schema for the workflow"""
    validate_payload: NotRequired[bool]
    r"""Enable or disable payload schema validation"""
    is_translation_enabled: NotRequired[bool]
    r"""Enable or disable translations for this workflow"""


class PatchWorkflowDto(BaseModel):
    active: Optional[bool] = None
    r"""Activate or deactivate the workflow"""

    name: Optional[str] = None
    r"""New name for the workflow"""

    description: Optional[str] = None
    r"""Updated description of the workflow"""

    tags: Optional[List[str]] = None
    r"""Tags associated with the workflow"""

    payload_schema: Annotated[
        OptionalNullable[Dict[str, Any]], pydantic.Field(alias="payloadSchema")
    ] = UNSET
    r"""The payload JSON Schema for the workflow"""

    validate_payload: Annotated[
        Optional[bool], pydantic.Field(alias="validatePayload")
    ] = None
    r"""Enable or disable payload schema validation"""

    is_translation_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="isTranslationEnabled")
    ] = None
    r"""Enable or disable translations for this workflow"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "active",
            "name",
            "description",
            "tags",
            "payloadSchema",
            "validatePayload",
            "isTranslationEnabled",
        ]
        nullable_fields = ["payloadSchema"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class EmailBlockTypeEnum(str, Enum):
    r"""Type of the email block"""

    BUTTON = "button"
    TEXT = "text"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .contentissueenum import ContentIssueEnum
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class StepContentIssueDtoTypedDict(TypedDict):
    issue_type: ContentIssueEnum
    r"""Type of step content issue"""
    message: str
    r"""Detailed message describing the issue"""
    variable_name: NotRequired[str]
    r"""Name of the variable related to the issue"""


class StepContentIssueDto(BaseModel):
    issue_type: Annotated[ContentIssueEnum, pydantic.Field(alias="issueType")]
    r"""Type of step content issue"""

    message: str
    r"""Detailed message describing the issue"""

    variable_name: Annotated[Optional[str], pydantic.Field(alias="variableName")] = None
    r"""Name of the variable related to the issue"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .lookbackwindowdto import LookBackWindowDto, LookBackWindowDtoTypedDict
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DigestControlDtoType(str, Enum):
    r"""The type of digest strategy. Determines which fields are applicable."""

    REGULAR = "regular"
    TIMED = "timed"


class DigestControlDtoUnit(str, Enum):
    r"""The unit of time for the digest interval (for REGULAR type)."""

    SECONDS = "seconds"
    MINUTES = "minutes"
    HOURS = "hours"
    DAYS = "days"
    WEEKS = "weeks"
    MONTHS = "months"


class DigestControlDtoTypedDict(TypedDict):
    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    type: NotRequired[DigestControlDtoType]
    r"""The type of digest strategy. Determines which fields are applicable."""
    amount: NotRequired[float]
    r"""The amount of time for the digest interval (for REGULAR type). Min 1."""
    unit: NotRequired[DigestControlDtoUnit]
    r"""The unit of time for the digest interval (for REGULAR type)."""
    look_back_window: NotRequired[LookBackWindowDtoTypedDict]
    r"""Configuration for look-back window (for REGULAR type)."""
    cron: NotRequired[str]
    r"""Cron expression for TIMED digest. Min length 1."""
    digest_key: NotRequired[str]
    r"""Specify a custom key for digesting events instead of the default event key."""


class DigestControlDto(BaseModel):
    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    type: Optional[DigestControlDtoType] = None
    r"""The type of digest strategy. Determines which fields are applicable."""

    amount: Optional[float] = None
    r"""The amount of time for the digest interval (for REGULAR type). Min 1."""

    unit: Optional[DigestControlDtoUnit] = None
    r"""The unit of time for the digest interval (for REGULAR type)."""

    look_back_window: Annotated[
        Optional[LookBackWindowDto], pydantic.Field(alias="lookBackWindow")
    ] = None
    r"""Configuration for look-back window (for REGULAR type)."""

    cron: Optional[str] = None
    r"""Cron expression for TIMED digest. Min length 1."""

    digest_key: Annotated[Optional[str], pydantic.Field(alias="digestKey")] = None
    r"""Specify a custom key for digesting events instead of the default event key."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from novu_py.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TranslationControllerGetMasterJSONEndpointRequestTypedDict(TypedDict):
    locale: NotRequired[str]
    r"""Locale to export. If not provided, exports organization default locale"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class TranslationControllerGetMasterJSONEndpointRequest(BaseModel):
    locale: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Locale to export. If not provided, exports organization default locale"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class NotificationTriggerVariableTypedDict(TypedDict):
    name: str
    r"""Name of the variable"""


class NotificationTriggerVariable(BaseModel):
    name: str
    r"""Name of the variable"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class ActivityTopicDtoTypedDict(TypedDict):
    topic_id: str
    r"""Internal Topic ID of the notification"""
    topic_key: str
    r"""Topic Key of the notification"""


class ActivityTopicDto(BaseModel):
    topic_id: Annotated[str, pydantic.Field(alias="_topicId")]
    r"""Internal Topic ID of the notification"""

    topic_key: Annotated[str, pydantic.Field(alias="topicKey")]
    r"""Topic Key of the notification"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .actorfeeditemdto import ActorFeedItemDto, ActorFeedItemDtoTypedDict
from .channeltypeenum import ChannelTypeEnum
from .messagecta import MessageCTA, MessageCTATypedDict
from .subscriberfeedresponsedto import (
    SubscriberFeedResponseDto,
    SubscriberFeedResponseDtoTypedDict,
)
from datetime import datetime
from enum import Enum
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class NotificationFeedItemDtoStatus(str, Enum):
    r"""Current status of the notification."""

    SENT = "sent"
    ERROR = "error"
    WARNING = "warning"


class NotificationFeedItemDtoTypedDict(TypedDict):
    id: str
    r"""Unique identifier for the notification."""
    template_id: str
    r"""Identifier for the template used to generate the notification."""
    environment_id: str
    r"""Identifier for the environment where the notification is sent."""
    organization_id: str
    r"""Identifier for the organization sending the notification."""
    notification_id: str
    r"""Unique identifier for the notification instance."""
    subscriber_id: str
    r"""Unique identifier for the subscriber receiving the notification."""
    job_id: str
    r"""Identifier for the job that triggered the notification."""
    transaction_id: str
    r"""Unique identifier for the transaction associated with the notification."""
    content: str
    r"""The main content of the notification."""
    channel: ChannelTypeEnum
    r"""Channel type through which the message is sent"""
    read: bool
    r"""Indicates whether the notification has been read by the subscriber."""
    seen: bool
    r"""Indicates whether the notification has been seen by the subscriber."""
    archived: bool
    r"""Indicates whether the notification has been archived by the subscriber."""
    cta: MessageCTATypedDict
    r"""Call-to-action information associated with the notification."""
    status: NotificationFeedItemDtoStatus
    r"""Current status of the notification."""
    message_template_id: NotRequired[str]
    r"""Identifier for the message template used."""
    feed_id: NotRequired[Nullable[str]]
    r"""Identifier for the feed associated with the notification."""
    created_at: NotRequired[Nullable[datetime]]
    r"""Timestamp indicating when the notification was created."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""Timestamp indicating when the notification was last updated."""
    actor: NotRequired[ActorFeedItemDtoTypedDict]
    r"""Actor details related to the notification, if applicable."""
    subscriber: NotRequired[SubscriberFeedResponseDtoTypedDict]
    r"""Subscriber details associated with this notification."""
    template_identifier: NotRequired[Nullable[str]]
    r"""Identifier for the template used, if applicable."""
    provider_id: NotRequired[Nullable[str]]
    r"""Identifier for the provider that sends the notification."""
    subject: NotRequired[Nullable[str]]
    r"""The subject line for email notifications, if applicable."""
    device_tokens: NotRequired[Nullable[List[str]]]
    r"""Device tokens for push notifications, if applicable."""
    payload: NotRequired[Dict[str, Any]]
    r"""The payload that was used to send the notification trigger."""
    data: NotRequired[Nullable[Dict[str, Any]]]
    r"""The data sent with the notification."""
    overrides: NotRequired[Dict[str, Any]]
    r"""Provider-specific overrides used when triggering the notification."""
    tags: NotRequired[Nullable[List[str]]]
    r"""Tags associated with the workflow that triggered the notification."""


class NotificationFeedItemDto(BaseModel):
    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Unique identifier for the notification."""

    template_id: Annotated[str, pydantic.Field(alias="_templateId")]
    r"""Identifier for the template used to generate the notification."""

    environment_id: Annotated[str, pydantic.Field(alias="_environmentId")]
    r"""Identifier for the environment where the notification is sent."""

    organization_id: Annotated[str, pydantic.Field(alias="_organizationId")]
    r"""Identifier for the organization sending the notification."""

    notification_id: Annotated[str, pydantic.Field(alias="_notificationId")]
    r"""Unique identifier for the notification instance."""

    subscriber_id: Annotated[str, pydantic.Field(alias="_subscriberId")]
    r"""Unique identifier for the subscriber receiving the notification."""

    job_id: Annotated[str, pydantic.Field(alias="_jobId")]
    r"""Identifier for the job that triggered the notification."""

    transaction_id: Annotated[str, pydantic.Field(alias="transactionId")]
    r"""Unique identifier for the transaction associated with the notification."""

    content: str
    r"""The main content of the notification."""

    channel: ChannelTypeEnum
    r"""Channel type through which the message is sent"""

    read: bool
    r"""Indicates whether the notification has been read by the subscriber."""

    seen: bool
    r"""Indicates whether the notification has been seen by the subscriber."""

    archived: bool
    r"""Indicates whether the notification has been archived by the subscriber."""

    cta: MessageCTA
    r"""Call-to-action information associated with the notification."""

    status: NotificationFeedItemDtoStatus
    r"""Current status of the notification."""

    message_template_id: Annotated[
        Optional[str], pydantic.Field(alias="_messageTemplateId")
    ] = None
    r"""Identifier for the message template used."""

    feed_id: Annotated[OptionalNullable[str], pydantic.Field(alias="_feedId")] = UNSET
    r"""Identifier for the feed associated with the notification."""

    created_at: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="createdAt")
    ] = UNSET
    r"""Timestamp indicating when the notification was created."""

    updated_at: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="updatedAt")
    ] = UNSET
    r"""Timestamp indicating when the notification was last updated."""

    actor: Optional[ActorFeedItemDto] = None
    r"""Actor details related to the notification, if applicable."""

    subscriber: Optional[SubscriberFeedResponseDto] = None
    r"""Subscriber details associated with this notification."""

    template_identifier: Annotated[
        OptionalNullable[str], pydantic.Field(alias="templateIdentifier")
    ] = UNSET
    r"""Identifier for the template used, if applicable."""

    provider_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="providerId")
    ] = UNSET
    r"""Identifier for the provider that sends the notification."""

    subject: OptionalNullable[str] = UNSET
    r"""The subject line for email notifications, if applicable."""

    device_tokens: Annotated[
        OptionalNullable[List[str]], pydantic.Field(alias="deviceTokens")
    ] = UNSET
    r"""Device tokens for push notifications, if applicable."""

    payload: Optional[Dict[str, Any]] = None
    r"""The payload that was used to send the notification trigger."""

    data: OptionalNullable[Dict[str, Any]] = UNSET
    r"""The data sent with the notification."""

    overrides: Optional[Dict[str, Any]] = None
    r"""Provider-specific overrides used when triggering the notification."""

    tags: OptionalNullable[List[str]] = UNSET
    r"""Tags associated with the workflow that triggered the notification."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "_messageTemplateId",
            "_feedId",
            "createdAt",
            "updatedAt",
            "actor",
            "subscriber",
            "templateIdentifier",
            "providerId",
            "subject",
            "deviceTokens",
            "payload",
            "data",
            "overrides",
            "tags",
        ]
        nullable_fields = [
            "_feedId",
            "createdAt",
            "updatedAt",
            "templateIdentifier",
            "providerId",
            "subject",
            "deviceTokens",
            "data",
            "tags",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TopicDtoTypedDict(TypedDict):
    id: str
    r"""The internal unique identifier of the topic"""
    key: str
    r"""The key identifier of the topic used in your application. Should be unique on the environment level."""
    name: NotRequired[str]
    r"""The name of the topic"""


class TopicDto(BaseModel):
    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""The internal unique identifier of the topic"""

    key: str
    r"""The key identifier of the topic used in your application. Should be unique on the environment level."""

    name: Optional[str] = None
    r"""The name of the topic"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CreateUpdateTopicRequestDtoTypedDict(TypedDict):
    key: str
    r"""The unique key identifier for the topic. The key must contain only alphanumeric characters (a-z, A-Z, 0-9), hyphens (-), underscores (_), colons (:), or be a valid email address."""
    name: NotRequired[str]
    r"""The display name for the topic"""


class CreateUpdateTopicRequestDto(BaseModel):
    key: str
    r"""The unique key identifier for the topic. The key must contain only alphanumeric characters (a-z, A-Z, 0-9), hyphens (-), underscores (_), colons (:), or be a valid email address."""

    name: Optional[str] = None
    r"""The display name for the topic"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .emailblockstyles import EmailBlockStyles, EmailBlockStylesTypedDict
from .emailblocktypeenum import EmailBlockTypeEnum
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EmailBlockTypedDict(TypedDict):
    type: EmailBlockTypeEnum
    r"""Type of the email block"""
    content: str
    r"""Content of the email block"""
    url: NotRequired[str]
    r"""URL associated with the email block, if any"""
    styles: NotRequired[EmailBlockStylesTypedDict]
    r"""Styles applied to the email block"""


class EmailBlock(BaseModel):
    type: EmailBlockTypeEnum
    r"""Type of the email block"""

    content: str
    r"""Content of the email block"""

    url: Optional[str] = None
    r"""URL associated with the email block, if any"""

    styles: Optional[EmailBlockStyles] = None
    r"""Styles applied to the email block"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .emaillayoutrenderoutput import (
    EmailLayoutRenderOutput,
    EmailLayoutRenderOutputTypedDict,
)
from .layoutpreviewpayloaddto import (
    LayoutPreviewPayloadDto,
    LayoutPreviewPayloadDtoTypedDict,
)
from enum import Enum
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ResultType(str, Enum):
    EMAIL = "email"


class OneTypedDict(TypedDict):
    type: NotRequired[ResultType]
    preview: NotRequired[EmailLayoutRenderOutputTypedDict]


class One(BaseModel):
    type: Optional[ResultType] = None

    preview: Optional[EmailLayoutRenderOutput] = None


ResultTypedDict = OneTypedDict
r"""Preview result"""


Result = One
r"""Preview result"""


class GenerateLayoutPreviewResponseDtoTypedDict(TypedDict):
    preview_payload_example: LayoutPreviewPayloadDtoTypedDict
    r"""Preview payload example"""
    result: ResultTypedDict
    r"""Preview result"""
    schema_: NotRequired[Nullable[Dict[str, Any]]]
    r"""The payload schema that was used to generate the preview payload example"""


class GenerateLayoutPreviewResponseDto(BaseModel):
    preview_payload_example: Annotated[
        LayoutPreviewPayloadDto, pydantic.Field(alias="previewPayloadExample")
    ]
    r"""Preview payload example"""

    result: Result
    r"""Preview result"""

    schema_: Annotated[
        OptionalNullable[Dict[str, Any]], pydantic.Field(alias="schema")
    ] = UNSET
    r"""The payload schema that was used to generate the preview payload example"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["schema"]
        nullable_fields = ["schema"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .controlsmetadatadto import ControlsMetadataDto, ControlsMetadataDtoTypedDict
from .resourceoriginenum import ResourceOriginEnum
from .stepissuesdto import StepIssuesDto, StepIssuesDtoTypedDict
from .steptypeenum import StepTypeEnum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class StepResponseDtoTypedDict(TypedDict):
    controls: ControlsMetadataDtoTypedDict
    r"""Controls metadata for the step"""
    variables: Dict[str, Any]
    r"""JSON Schema for variables, follows the JSON Schema standard"""
    step_id: str
    r"""Unique identifier of the step"""
    id: str
    r"""Database identifier of the step"""
    name: str
    r"""Name of the step"""
    slug: str
    r"""Slug of the step"""
    type: StepTypeEnum
    r"""Type of the step"""
    origin: ResourceOriginEnum
    r"""Origin of the layout"""
    workflow_id: str
    r"""Workflow identifier"""
    workflow_database_id: str
    r"""Workflow database identifier"""
    control_values: NotRequired[Dict[str, Any]]
    r"""Control values for the step (alias for controls.values)"""
    issues: NotRequired[StepIssuesDtoTypedDict]
    r"""Issues associated with the step"""


class StepResponseDto(BaseModel):
    controls: ControlsMetadataDto
    r"""Controls metadata for the step"""

    variables: Dict[str, Any]
    r"""JSON Schema for variables, follows the JSON Schema standard"""

    step_id: Annotated[str, pydantic.Field(alias="stepId")]
    r"""Unique identifier of the step"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Database identifier of the step"""

    name: str
    r"""Name of the step"""

    slug: str
    r"""Slug of the step"""

    type: StepTypeEnum
    r"""Type of the step"""

    origin: ResourceOriginEnum
    r"""Origin of the layout"""

    workflow_id: Annotated[str, pydantic.Field(alias="workflowId")]
    r"""Workflow identifier"""

    workflow_database_id: Annotated[str, pydantic.Field(alias="workflowDatabaseId")]
    r"""Workflow database identifier"""

    control_values: Annotated[
        Optional[Dict[str, Any]], pydantic.Field(alias="controlValues")
    ] = None
    r"""Control values for the step (alias for controls.values)"""

    issues: Optional[StepIssuesDto] = None
    r"""Issues associated with the step"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .delayregularmetadata import DelayRegularMetadata, DelayRegularMetadataTypedDict
from .delayscheduledmetadata import (
    DelayScheduledMetadata,
    DelayScheduledMetadataTypedDict,
)
from .digestregularmetadata import DigestRegularMetadata, DigestRegularMetadataTypedDict
from .digesttimedmetadata import DigestTimedMetadata, DigestTimedMetadataTypedDict
from .messagetemplate import MessageTemplate, MessageTemplateTypedDict
from .replycallback import ReplyCallback, ReplyCallbackTypedDict
from .stepfilterdto import StepFilterDto, StepFilterDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


NotificationStepDataMetadataTypedDict = TypeAliasType(
    "NotificationStepDataMetadataTypedDict",
    Union[
        DelayScheduledMetadataTypedDict,
        DelayRegularMetadataTypedDict,
        DigestTimedMetadataTypedDict,
        DigestRegularMetadataTypedDict,
    ],
)
r"""Metadata associated with the workflow step. Can vary based on the type of step."""


NotificationStepDataMetadata = TypeAliasType(
    "NotificationStepDataMetadata",
    Union[
        DelayScheduledMetadata,
        DelayRegularMetadata,
        DigestTimedMetadata,
        DigestRegularMetadata,
    ],
)
r"""Metadata associated with the workflow step. Can vary based on the type of step."""


class NotificationStepDataTypedDict(TypedDict):
    id: NotRequired[str]
    r"""Unique identifier for the notification step."""
    uuid: NotRequired[str]
    r"""Universally unique identifier for the notification step."""
    name: NotRequired[str]
    r"""Name of the notification step."""
    template_id: NotRequired[str]
    r"""ID of the template associated with this notification step."""
    active: NotRequired[bool]
    r"""Indicates whether the notification step is active."""
    should_stop_on_fail: NotRequired[bool]
    r"""Determines if the process should stop on failure."""
    template: NotRequired[MessageTemplateTypedDict]
    r"""Message template used in this notification step."""
    filters: NotRequired[List[StepFilterDtoTypedDict]]
    r"""Filters applied to this notification step."""
    parent_id: NotRequired[str]
    r"""ID of the parent notification step, if applicable."""
    metadata: NotRequired[NotificationStepDataMetadataTypedDict]
    r"""Metadata associated with the workflow step. Can vary based on the type of step."""
    reply_callback: NotRequired[ReplyCallbackTypedDict]
    r"""Callback information for replies, including whether it is active and the callback URL."""


class NotificationStepData(BaseModel):
    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""Unique identifier for the notification step."""

    uuid: Optional[str] = None
    r"""Universally unique identifier for the notification step."""

    name: Optional[str] = None
    r"""Name of the notification step."""

    template_id: Annotated[Optional[str], pydantic.Field(alias="_templateId")] = None
    r"""ID of the template associated with this notification step."""

    active: Optional[bool] = None
    r"""Indicates whether the notification step is active."""

    should_stop_on_fail: Annotated[
        Optional[bool], pydantic.Field(alias="shouldStopOnFail")
    ] = None
    r"""Determines if the process should stop on failure."""

    template: Optional[MessageTemplate] = None
    r"""Message template used in this notification step."""

    filters: Optional[List[StepFilterDto]] = None
    r"""Filters applied to this notification step."""

    parent_id: Annotated[Optional[str], pydantic.Field(alias="_parentId")] = None
    r"""ID of the parent notification step, if applicable."""

    metadata: Optional[NotificationStepDataMetadata] = None
    r"""Metadata associated with the workflow step. Can vary based on the type of step."""

    reply_callback: Annotated[
        Optional[ReplyCallback], pydantic.Field(alias="replyCallback")
    ] = None
    r"""Callback information for replies, including whether it is active and the callback URL."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SeverityLevelEnum(str, Enum):
    r"""Severity of the workflow"""

    HIGH = "high"
    MEDIUM = "medium"
    LOW = "low"
    NONE = "none"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DigestUnitEnum(str, Enum):
    r"""Regular digest: Unit for backoff"""

    SECONDS = "seconds"
    MINUTES = "minutes"
    HOURS = "hours"
    DAYS = "days"
    WEEKS = "weeks"
    MONTHS = "months"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ResourceTypeEnum(str, Enum):
    r"""Type of the layout"""

    REGULAR = "REGULAR"
    ECHO = "ECHO"
    BRIDGE = "BRIDGE"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class MessagesControllerDeleteMessagesByTransactionIDQueryParamChannel(str, Enum):
    r"""The channel of the message to be deleted"""

    IN_APP = "in_app"
    EMAIL = "email"
    SMS = "sms"
    CHAT = "chat"
    PUSH = "push"


class MessagesControllerDeleteMessagesByTransactionIDRequestTypedDict(TypedDict):
    transaction_id: str
    channel: NotRequired[
        MessagesControllerDeleteMessagesByTransactionIDQueryParamChannel
    ]
    r"""The channel of the message to be deleted"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class MessagesControllerDeleteMessagesByTransactionIDRequest(BaseModel):
    transaction_id: Annotated[
        str,
        pydantic.Field(alias="transactionId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    channel: Annotated[
        Optional[MessagesControllerDeleteMessagesByTransactionIDQueryParamChannel],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The channel of the message to be deleted"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class MessagesControllerDeleteMessagesByTransactionIDResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]


class MessagesControllerDeleteMessagesByTransactionIDResponse(BaseModel):
    headers: Dict[str, List[str]]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .uischemagroupenum import UISchemaGroupEnum
from .uischemaproperty import UISchemaProperty, UISchemaPropertyTypedDict
from novu_py.types import BaseModel
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class UISchemaTypedDict(TypedDict):
    group: NotRequired[UISchemaGroupEnum]
    r"""Group of the UI Schema"""
    properties: NotRequired[Dict[str, UISchemaPropertyTypedDict]]
    r"""Properties of the UI Schema"""


class UISchema(BaseModel):
    group: Optional[UISchemaGroupEnum] = None
    r"""Group of the UI Schema"""

    properties: Optional[Dict[str, UISchemaProperty]] = None
    r"""Properties of the UI Schema"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createsubscriberrequestdto import (
    CreateSubscriberRequestDto,
    CreateSubscriberRequestDtoTypedDict,
)
from novu_py.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class BulkSubscriberCreateDtoTypedDict(TypedDict):
    subscribers: List[CreateSubscriberRequestDtoTypedDict]
    r"""An array of subscribers to be created in bulk."""


class BulkSubscriberCreateDto(BaseModel):
    subscribers: List[CreateSubscriberRequestDto]
    r"""An array of subscribers to be created in bulk."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DigestRegularMetadataUnit(str, Enum):
    SECONDS = "seconds"
    MINUTES = "minutes"
    HOURS = "hours"
    DAYS = "days"
    WEEKS = "weeks"
    MONTHS = "months"


class DigestRegularMetadataType(str, Enum):
    REGULAR = "regular"
    BACKOFF = "backoff"


class BackoffUnit(str, Enum):
    SECONDS = "seconds"
    MINUTES = "minutes"
    HOURS = "hours"
    DAYS = "days"
    WEEKS = "weeks"
    MONTHS = "months"


class DigestRegularMetadataTypedDict(TypedDict):
    type: DigestRegularMetadataType
    amount: NotRequired[float]
    unit: NotRequired[DigestRegularMetadataUnit]
    digest_key: NotRequired[str]
    backoff: NotRequired[bool]
    backoff_amount: NotRequired[float]
    backoff_unit: NotRequired[BackoffUnit]
    update_mode: NotRequired[bool]


class DigestRegularMetadata(BaseModel):
    type: DigestRegularMetadataType

    amount: Optional[float] = None

    unit: Optional[DigestRegularMetadataUnit] = None

    digest_key: Annotated[Optional[str], pydantic.Field(alias="digestKey")] = None

    backoff: Optional[bool] = None

    backoff_amount: Annotated[
        Optional[float], pydantic.Field(alias="backoffAmount")
    ] = None

    backoff_unit: Annotated[
        Optional[BackoffUnit], pydantic.Field(alias="backoffUnit")
    ] = None

    update_mode: Annotated[Optional[bool], pydantic.Field(alias="updateMode")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class TopicSubscriberDtoTypedDict(TypedDict):
    organization_id: str
    r"""Unique identifier for the organization"""
    environment_id: str
    r"""Unique identifier for the environment"""
    subscriber_id: str
    r"""Unique identifier for the subscriber"""
    topic_id: str
    r"""Unique identifier for the topic"""
    topic_key: str
    r"""Key associated with the topic"""
    external_subscriber_id: str
    r"""External identifier for the subscriber"""


class TopicSubscriberDto(BaseModel):
    organization_id: Annotated[str, pydantic.Field(alias="_organizationId")]
    r"""Unique identifier for the organization"""

    environment_id: Annotated[str, pydantic.Field(alias="_environmentId")]
    r"""Unique identifier for the environment"""

    subscriber_id: Annotated[str, pydantic.Field(alias="_subscriberId")]
    r"""Unique identifier for the subscriber"""

    topic_id: Annotated[str, pydantic.Field(alias="_topicId")]
    r"""Unique identifier for the topic"""

    topic_key: Annotated[str, pydantic.Field(alias="topicKey")]
    r"""Key associated with the topic"""

    external_subscriber_id: Annotated[str, pydantic.Field(alias="externalSubscriberId")]
    r"""External identifier for the subscriber"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bulkupdatesubscriberpreferenceitemdto import (
    BulkUpdateSubscriberPreferenceItemDto,
    BulkUpdateSubscriberPreferenceItemDtoTypedDict,
)
from novu_py.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class BulkUpdateSubscriberPreferencesDtoTypedDict(TypedDict):
    preferences: List[BulkUpdateSubscriberPreferenceItemDtoTypedDict]
    r"""Array of workflow preferences to update (maximum 100 items)"""


class BulkUpdateSubscriberPreferencesDto(BaseModel):
    preferences: List[BulkUpdateSubscriberPreferenceItemDto]
    r"""Array of workflow preferences to update (maximum 100 items)"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channelpreferencedto import ChannelPreferenceDto, ChannelPreferenceDtoTypedDict
from .workflowpreferencedto import WorkflowPreferenceDto, WorkflowPreferenceDtoTypedDict
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict
from typing_extensions import NotRequired, TypedDict


UserAllTypedDict = WorkflowPreferenceDtoTypedDict
r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""


UserAll = WorkflowPreferenceDto
r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""


class UserWorkflowPreferencesDtoTypedDict(TypedDict):
    all: UserAllTypedDict
    r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""
    channels: Dict[str, ChannelPreferenceDtoTypedDict]
    r"""Preferences for different communication channels"""


class UserWorkflowPreferencesDto(BaseModel):
    all: UserAll
    r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""

    channels: Dict[str, ChannelPreferenceDto]
    r"""Preferences for different communication channels"""


UserTypedDict = UserWorkflowPreferencesDtoTypedDict
r"""User workflow preferences"""


User = UserWorkflowPreferencesDto
r"""User workflow preferences"""


PreferencesRequestDtoAllTypedDict = WorkflowPreferenceDtoTypedDict
r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""


PreferencesRequestDtoAll = WorkflowPreferenceDto
r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""


class PreferencesRequestDtoWorkflowTypedDict(TypedDict):
    r"""Workflow-specific preferences"""

    all: PreferencesRequestDtoAllTypedDict
    r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""
    channels: Dict[str, ChannelPreferenceDtoTypedDict]
    r"""Preferences for different communication channels"""


class PreferencesRequestDtoWorkflow(BaseModel):
    r"""Workflow-specific preferences"""

    all: PreferencesRequestDtoAll
    r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""

    channels: Dict[str, ChannelPreferenceDto]
    r"""Preferences for different communication channels"""


class PreferencesRequestDtoTypedDict(TypedDict):
    user: NotRequired[Nullable[UserTypedDict]]
    r"""User workflow preferences"""
    workflow: NotRequired[Nullable[PreferencesRequestDtoWorkflowTypedDict]]
    r"""Workflow-specific preferences"""


class PreferencesRequestDto(BaseModel):
    user: OptionalNullable[User] = UNSET
    r"""User workflow preferences"""

    workflow: OptionalNullable[PreferencesRequestDtoWorkflow] = UNSET
    r"""Workflow-specific preferences"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["user", "workflow"]
        nullable_fields = ["user", "workflow"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .monthlytypeenum import MonthlyTypeEnum
from .ordinalenum import OrdinalEnum
from .ordinalvalueenum import OrdinalValueEnum
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DigestTimedConfigDtoWeekDays(str, Enum):
    MONDAY = "monday"
    TUESDAY = "tuesday"
    WEDNESDAY = "wednesday"
    THURSDAY = "thursday"
    FRIDAY = "friday"
    SATURDAY = "saturday"
    SUNDAY = "sunday"


class DigestTimedConfigDtoTypedDict(TypedDict):
    at_time: NotRequired[str]
    r"""Time at which the digest is triggered"""
    week_days: NotRequired[List[DigestTimedConfigDtoWeekDays]]
    r"""Days of the week for the digest"""
    month_days: NotRequired[List[float]]
    r"""Specific days of the month for the digest"""
    ordinal: NotRequired[OrdinalEnum]
    r"""Ordinal position for the digest"""
    ordinal_value: NotRequired[OrdinalValueEnum]
    r"""Value of the ordinal"""
    monthly_type: NotRequired[MonthlyTypeEnum]
    r"""Type of monthly schedule"""
    cron_expression: NotRequired[str]
    r"""Cron expression for scheduling"""
    until_date: NotRequired[str]
    r"""Until date for scheduling"""


class DigestTimedConfigDto(BaseModel):
    at_time: Annotated[Optional[str], pydantic.Field(alias="atTime")] = None
    r"""Time at which the digest is triggered"""

    week_days: Annotated[
        Optional[List[DigestTimedConfigDtoWeekDays]], pydantic.Field(alias="weekDays")
    ] = None
    r"""Days of the week for the digest"""

    month_days: Annotated[Optional[List[float]], pydantic.Field(alias="monthDays")] = (
        None
    )
    r"""Specific days of the month for the digest"""

    ordinal: Optional[OrdinalEnum] = None
    r"""Ordinal position for the digest"""

    ordinal_value: Annotated[
        Optional[OrdinalValueEnum], pydantic.Field(alias="ordinalValue")
    ] = None
    r"""Value of the ordinal"""

    monthly_type: Annotated[
        Optional[MonthlyTypeEnum], pydantic.Field(alias="monthlyType")
    ] = None
    r"""Type of monthly schedule"""

    cron_expression: Annotated[
        Optional[str], pydantic.Field(alias="cronExpression")
    ] = None
    r"""Cron expression for scheduling"""

    until_date: Annotated[Optional[str], pydantic.Field(alias="untilDate")] = None
    r"""Until date for scheduling"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .notificationfeeditemdto import (
    NotificationFeedItemDto,
    NotificationFeedItemDtoTypedDict,
)
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FeedResponseDtoTypedDict(TypedDict):
    has_more: bool
    r"""Indicates if there are more notifications to load."""
    data: List[NotificationFeedItemDtoTypedDict]
    r"""Array of notifications returned in the response."""
    page_size: float
    r"""The number of notifications returned in this response."""
    page: float
    r"""The current page number of the notifications."""
    total_count: NotRequired[float]
    r"""Total number of notifications available."""


class FeedResponseDto(BaseModel):
    has_more: Annotated[bool, pydantic.Field(alias="hasMore")]
    r"""Indicates if there are more notifications to load."""

    data: List[NotificationFeedItemDto]
    r"""Array of notifications returned in the response."""

    page_size: Annotated[float, pydantic.Field(alias="pageSize")]
    r"""The number of notifications returned in this response."""

    page: float
    r"""The current page number of the notifications."""

    total_count: Annotated[Optional[float], pydantic.Field(alias="totalCount")] = None
    r"""Total number of notifications available."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .directionenum import DirectionEnum
from .listworkflowresponse import ListWorkflowResponse, ListWorkflowResponseTypedDict
from .workflowresponsedtosortfield import WorkflowResponseDtoSortField
from .workflowstatusenum import WorkflowStatusEnum
from novu_py.types import BaseModel
from novu_py.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WorkflowControllerSearchWorkflowsRequestTypedDict(TypedDict):
    limit: NotRequired[float]
    r"""Number of items to return per page"""
    offset: NotRequired[float]
    r"""Number of items to skip before starting to return results"""
    order_direction: NotRequired[DirectionEnum]
    r"""Direction of sorting"""
    order_by: NotRequired[WorkflowResponseDtoSortField]
    r"""Field to sort the results by"""
    query: NotRequired[str]
    r"""Search query to filter workflows"""
    tags: NotRequired[List[str]]
    r"""Filter workflows by tags"""
    status: NotRequired[List[WorkflowStatusEnum]]
    r"""Filter workflows by status"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class WorkflowControllerSearchWorkflowsRequest(BaseModel):
    limit: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Number of items to return per page"""

    offset: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Number of items to skip before starting to return results"""

    order_direction: Annotated[
        Optional[DirectionEnum],
        pydantic.Field(alias="orderDirection"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Direction of sorting"""

    order_by: Annotated[
        Optional[WorkflowResponseDtoSortField],
        pydantic.Field(alias="orderBy"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Field to sort the results by"""

    query: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Search query to filter workflows"""

    tags: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter workflows by tags"""

    status: Annotated[
        Optional[List[WorkflowStatusEnum]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter workflows by status"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class WorkflowControllerSearchWorkflowsResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: ListWorkflowResponseTypedDict


class WorkflowControllerSearchWorkflowsResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: ListWorkflowResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createmsteamschannelendpointdto import (
    CreateMsTeamsChannelEndpointDto,
    CreateMsTeamsChannelEndpointDtoTypedDict,
)
from .createmsteamsuserendpointdto import (
    CreateMsTeamsUserEndpointDto,
    CreateMsTeamsUserEndpointDtoTypedDict,
)
from .createphoneendpointdto import (
    CreatePhoneEndpointDto,
    CreatePhoneEndpointDtoTypedDict,
)
from .createslackchannelendpointdto import (
    CreateSlackChannelEndpointDto,
    CreateSlackChannelEndpointDtoTypedDict,
)
from .createslackuserendpointdto import (
    CreateSlackUserEndpointDto,
    CreateSlackUserEndpointDtoTypedDict,
)
from .createwebhookendpointdto import (
    CreateWebhookEndpointDto,
    CreateWebhookEndpointDtoTypedDict,
)
from .getchannelendpointresponsedto import (
    GetChannelEndpointResponseDto,
    GetChannelEndpointResponseDtoTypedDict,
)
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    RequestMetadata,
    get_discriminator,
)
import pydantic
from pydantic import Discriminator, Tag
from typing import Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ChannelEndpointsControllerCreateChannelEndpointRequestBodyTypedDict = TypeAliasType(
    "ChannelEndpointsControllerCreateChannelEndpointRequestBodyTypedDict",
    Union[
        CreateSlackChannelEndpointDtoTypedDict,
        CreateSlackUserEndpointDtoTypedDict,
        CreateWebhookEndpointDtoTypedDict,
        CreatePhoneEndpointDtoTypedDict,
        CreateMsTeamsChannelEndpointDtoTypedDict,
        CreateMsTeamsUserEndpointDtoTypedDict,
    ],
)
r"""Channel endpoint creation request. The structure varies based on the type field."""


ChannelEndpointsControllerCreateChannelEndpointRequestBody = Annotated[
    Union[
        Annotated[CreateSlackChannelEndpointDto, Tag("slack_channel")],
        Annotated[CreateSlackUserEndpointDto, Tag("slack_user")],
        Annotated[CreateWebhookEndpointDto, Tag("webhook")],
        Annotated[CreatePhoneEndpointDto, Tag("phone")],
        Annotated[CreateMsTeamsChannelEndpointDto, Tag("ms_teams_channel")],
        Annotated[CreateMsTeamsUserEndpointDto, Tag("ms_teams_user")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]
r"""Channel endpoint creation request. The structure varies based on the type field."""


class ChannelEndpointsControllerCreateChannelEndpointRequestTypedDict(TypedDict):
    request_body: ChannelEndpointsControllerCreateChannelEndpointRequestBodyTypedDict
    r"""Channel endpoint creation request. The structure varies based on the type field."""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class ChannelEndpointsControllerCreateChannelEndpointRequest(BaseModel):
    request_body: Annotated[
        ChannelEndpointsControllerCreateChannelEndpointRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Channel endpoint creation request. The structure varies based on the type field."""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class ChannelEndpointsControllerCreateChannelEndpointResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: GetChannelEndpointResponseDtoTypedDict


class ChannelEndpointsControllerCreateChannelEndpointResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: GetChannelEndpointResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ChannelPreferenceDtoTypedDict(TypedDict):
    enabled: NotRequired[bool]
    r"""A flag specifying if notification delivery is enabled for the channel. If true, notification delivery is enabled."""


class ChannelPreferenceDto(BaseModel):
    enabled: Optional[bool] = True
    r"""A flag specifying if notification delivery is enabled for the channel. If true, notification delivery is enabled."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .delayregularmetadata import DelayRegularMetadata, DelayRegularMetadataTypedDict
from .delayscheduledmetadata import (
    DelayScheduledMetadata,
    DelayScheduledMetadataTypedDict,
)
from .digestregularmetadata import DigestRegularMetadata, DigestRegularMetadataTypedDict
from .digesttimedmetadata import DigestTimedMetadata, DigestTimedMetadataTypedDict
from .messagetemplate import MessageTemplate, MessageTemplateTypedDict
from .notificationstepdata import NotificationStepData, NotificationStepDataTypedDict
from .replycallback import ReplyCallback, ReplyCallbackTypedDict
from .stepfilterdto import StepFilterDto, StepFilterDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


MetadataTypedDict = TypeAliasType(
    "MetadataTypedDict",
    Union[
        DelayScheduledMetadataTypedDict,
        DelayRegularMetadataTypedDict,
        DigestTimedMetadataTypedDict,
        DigestRegularMetadataTypedDict,
    ],
)
r"""Metadata associated with the workflow step. Can vary based on the type of step."""


Metadata = TypeAliasType(
    "Metadata",
    Union[
        DelayScheduledMetadata,
        DelayRegularMetadata,
        DigestTimedMetadata,
        DigestRegularMetadata,
    ],
)
r"""Metadata associated with the workflow step. Can vary based on the type of step."""


class NotificationStepDtoTypedDict(TypedDict):
    id: NotRequired[str]
    r"""Unique identifier for the notification step."""
    uuid: NotRequired[str]
    r"""Universally unique identifier for the notification step."""
    name: NotRequired[str]
    r"""Name of the notification step."""
    template_id: NotRequired[str]
    r"""ID of the template associated with this notification step."""
    active: NotRequired[bool]
    r"""Indicates whether the notification step is active."""
    should_stop_on_fail: NotRequired[bool]
    r"""Determines if the process should stop on failure."""
    template: NotRequired[MessageTemplateTypedDict]
    r"""Message template used in this notification step."""
    filters: NotRequired[List[StepFilterDtoTypedDict]]
    r"""Filters applied to this notification step."""
    parent_id: NotRequired[str]
    r"""ID of the parent notification step, if applicable."""
    metadata: NotRequired[MetadataTypedDict]
    r"""Metadata associated with the workflow step. Can vary based on the type of step."""
    reply_callback: NotRequired[ReplyCallbackTypedDict]
    r"""Callback information for replies, including whether it is active and the callback URL."""
    variants: NotRequired[List[NotificationStepDataTypedDict]]


class NotificationStepDto(BaseModel):
    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""Unique identifier for the notification step."""

    uuid: Optional[str] = None
    r"""Universally unique identifier for the notification step."""

    name: Optional[str] = None
    r"""Name of the notification step."""

    template_id: Annotated[Optional[str], pydantic.Field(alias="_templateId")] = None
    r"""ID of the template associated with this notification step."""

    active: Optional[bool] = None
    r"""Indicates whether the notification step is active."""

    should_stop_on_fail: Annotated[
        Optional[bool], pydantic.Field(alias="shouldStopOnFail")
    ] = None
    r"""Determines if the process should stop on failure."""

    template: Optional[MessageTemplate] = None
    r"""Message template used in this notification step."""

    filters: Optional[List[StepFilterDto]] = None
    r"""Filters applied to this notification step."""

    parent_id: Annotated[Optional[str], pydantic.Field(alias="_parentId")] = None
    r"""ID of the parent notification step, if applicable."""

    metadata: Optional[Metadata] = None
    r"""Metadata associated with the workflow step. Can vary based on the type of step."""

    reply_callback: Annotated[
        Optional[ReplyCallback], pydantic.Field(alias="replyCallback")
    ] = None
    r"""Callback information for replies, including whether it is active and the callback URL."""

    variants: Optional[List[NotificationStepData]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriberglobalpreferencedto import (
    SubscriberGlobalPreferenceDto,
    SubscriberGlobalPreferenceDtoTypedDict,
)
from .subscriberworkflowpreferencedto import (
    SubscriberWorkflowPreferenceDto,
    SubscriberWorkflowPreferenceDtoTypedDict,
)
from novu_py.types import BaseModel
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class GetSubscriberPreferencesDtoTypedDict(TypedDict):
    global_: SubscriberGlobalPreferenceDtoTypedDict
    r"""Global preference settings"""
    workflows: List[SubscriberWorkflowPreferenceDtoTypedDict]
    r"""Workflow-specific preference settings"""


class GetSubscriberPreferencesDto(BaseModel):
    global_: Annotated[SubscriberGlobalPreferenceDto, pydantic.Field(alias="global")]
    r"""Global preference settings"""

    workflows: List[SubscriberWorkflowPreferenceDto]
    r"""Workflow-specific preference settings"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .buttontypeenum import ButtonTypeEnum
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class MessageButtonTypedDict(TypedDict):
    type: ButtonTypeEnum
    r"""Type of button for the action result"""
    content: str
    r"""Content of the button"""
    result_content: NotRequired[str]
    r"""Content of the result when the button is clicked"""


class MessageButton(BaseModel):
    type: ButtonTypeEnum
    r"""Type of button for the action result"""

    content: str
    r"""Content of the button"""

    result_content: Annotated[Optional[str], pydantic.Field(alias="resultContent")] = (
        None
    )
    r"""Content of the result when the button is clicked"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .msteamsuserendpointdto import (
    MsTeamsUserEndpointDto,
    MsTeamsUserEndpointDtoTypedDict,
)
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class CreateMsTeamsUserEndpointDtoContext2TypedDict(TypedDict):
    r"""Rich context object with id and optional data"""

    id: str
    data: NotRequired[Dict[str, Any]]
    r"""Optional additional context data"""


class CreateMsTeamsUserEndpointDtoContext2(BaseModel):
    r"""Rich context object with id and optional data"""

    id: str

    data: Optional[Dict[str, Any]] = None
    r"""Optional additional context data"""


CreateMsTeamsUserEndpointDtoContextTypedDict = TypeAliasType(
    "CreateMsTeamsUserEndpointDtoContextTypedDict",
    Union[CreateMsTeamsUserEndpointDtoContext2TypedDict, str],
)


CreateMsTeamsUserEndpointDtoContext = TypeAliasType(
    "CreateMsTeamsUserEndpointDtoContext",
    Union[CreateMsTeamsUserEndpointDtoContext2, str],
)


class CreateMsTeamsUserEndpointDtoType(str, Enum):
    r"""Type of channel endpoint"""

    MS_TEAMS_USER = "ms_teams_user"


class CreateMsTeamsUserEndpointDtoTypedDict(TypedDict):
    subscriber_id: str
    r"""The subscriber ID to which the channel endpoint is linked"""
    integration_identifier: str
    r"""The identifier of the integration to use for this channel endpoint."""
    type: CreateMsTeamsUserEndpointDtoType
    r"""Type of channel endpoint"""
    endpoint: MsTeamsUserEndpointDtoTypedDict
    r"""MS Teams user endpoint data"""
    identifier: NotRequired[str]
    r"""The unique identifier for the channel endpoint. If not provided, one will be generated automatically."""
    context: NotRequired[Dict[str, CreateMsTeamsUserEndpointDtoContextTypedDict]]
    connection_identifier: NotRequired[str]
    r"""The identifier of the channel connection to use for this channel endpoint."""


class CreateMsTeamsUserEndpointDto(BaseModel):
    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""The subscriber ID to which the channel endpoint is linked"""

    integration_identifier: Annotated[
        str, pydantic.Field(alias="integrationIdentifier")
    ]
    r"""The identifier of the integration to use for this channel endpoint."""

    type: CreateMsTeamsUserEndpointDtoType
    r"""Type of channel endpoint"""

    endpoint: MsTeamsUserEndpointDto
    r"""MS Teams user endpoint data"""

    identifier: Optional[str] = None
    r"""The unique identifier for the channel endpoint. If not provided, one will be generated automatically."""

    context: Optional[Dict[str, CreateMsTeamsUserEndpointDtoContext]] = None

    connection_identifier: Annotated[
        Optional[str], pydantic.Field(alias="connectionIdentifier")
    ] = None
    r"""The identifier of the channel connection to use for this channel endpoint."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .emailcontrolsmetadataresponsedto import (
    EmailControlsMetadataResponseDto,
    EmailControlsMetadataResponseDtoTypedDict,
)
from .resourceoriginenum import ResourceOriginEnum
from .stepissuesdto import StepIssuesDto, StepIssuesDtoTypedDict
from .steptypeenum import StepTypeEnum
from enum import Enum
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import ConfigDict, model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EmailStepResponseDtoEditorType(str, Enum):
    r"""Type of editor to use for the body."""

    BLOCK = "block"
    HTML = "html"


class EmailStepResponseDtoControlValuesTypedDict(TypedDict):
    r"""Control values for the email step"""

    subject: str
    r"""Subject of the email."""
    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    body: NotRequired[str]
    r"""Body content of the email, either a valid Maily JSON object, or html string."""
    editor_type: NotRequired[EmailStepResponseDtoEditorType]
    r"""Type of editor to use for the body."""
    disable_output_sanitization: NotRequired[bool]
    r"""Disable sanitization of the output."""
    layout_id: NotRequired[Nullable[str]]
    r"""Layout ID to use for the email. Null means no layout, undefined means default layout."""


class EmailStepResponseDtoControlValues(BaseModel):
    r"""Control values for the email step"""

    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, extra="allow"
    )
    __pydantic_extra__: Dict[str, Any] = pydantic.Field(init=False)

    subject: str
    r"""Subject of the email."""

    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    body: Optional[str] = ""
    r"""Body content of the email, either a valid Maily JSON object, or html string."""

    editor_type: Annotated[
        Optional[EmailStepResponseDtoEditorType], pydantic.Field(alias="editorType")
    ] = EmailStepResponseDtoEditorType.BLOCK
    r"""Type of editor to use for the body."""

    disable_output_sanitization: Annotated[
        Optional[bool], pydantic.Field(alias="disableOutputSanitization")
    ] = False
    r"""Disable sanitization of the output."""

    layout_id: Annotated[OptionalNullable[str], pydantic.Field(alias="layoutId")] = (
        UNSET
    )
    r"""Layout ID to use for the email. Null means no layout, undefined means default layout."""

    @property
    def additional_properties(self):
        return self.__pydantic_extra__

    @additional_properties.setter
    def additional_properties(self, value):
        self.__pydantic_extra__ = value  # pyright: ignore[reportIncompatibleVariableOverride]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "skip",
            "body",
            "editorType",
            "disableOutputSanitization",
            "layoutId",
        ]
        nullable_fields = ["layoutId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        for k, v in serialized.items():
            m[k] = v

        return m


class EmailStepResponseDtoTypedDict(TypedDict):
    controls: EmailControlsMetadataResponseDtoTypedDict
    r"""Controls metadata for the email step"""
    variables: Dict[str, Any]
    r"""JSON Schema for variables, follows the JSON Schema standard"""
    step_id: str
    r"""Unique identifier of the step"""
    id: str
    r"""Database identifier of the step"""
    name: str
    r"""Name of the step"""
    slug: str
    r"""Slug of the step"""
    type: StepTypeEnum
    r"""Type of the step"""
    origin: ResourceOriginEnum
    r"""Origin of the layout"""
    workflow_id: str
    r"""Workflow identifier"""
    workflow_database_id: str
    r"""Workflow database identifier"""
    control_values: NotRequired[EmailStepResponseDtoControlValuesTypedDict]
    r"""Control values for the email step"""
    issues: NotRequired[StepIssuesDtoTypedDict]
    r"""Issues associated with the step"""


class EmailStepResponseDto(BaseModel):
    controls: EmailControlsMetadataResponseDto
    r"""Controls metadata for the email step"""

    variables: Dict[str, Any]
    r"""JSON Schema for variables, follows the JSON Schema standard"""

    step_id: Annotated[str, pydantic.Field(alias="stepId")]
    r"""Unique identifier of the step"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Database identifier of the step"""

    name: str
    r"""Name of the step"""

    slug: str
    r"""Slug of the step"""

    type: StepTypeEnum
    r"""Type of the step"""

    origin: ResourceOriginEnum
    r"""Origin of the layout"""

    workflow_id: Annotated[str, pydantic.Field(alias="workflowId")]
    r"""Workflow identifier"""

    workflow_database_id: Annotated[str, pydantic.Field(alias="workflowDatabaseId")]
    r"""Workflow database identifier"""

    control_values: Annotated[
        Optional[EmailStepResponseDtoControlValues],
        pydantic.Field(alias="controlValues"),
    ] = None
    r"""Control values for the email step"""

    issues: Optional[StepIssuesDto] = None
    r"""Issues associated with the step"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriberpreferencechannels import (
    SubscriberPreferenceChannels,
    SubscriberPreferenceChannelsTypedDict,
)
from .subscriberpreferenceoverridedto import (
    SubscriberPreferenceOverrideDto,
    SubscriberPreferenceOverrideDtoTypedDict,
)
from .subscriberpreferencesworkflowinfodto import (
    SubscriberPreferencesWorkflowInfoDto,
    SubscriberPreferencesWorkflowInfoDtoTypedDict,
)
from novu_py.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class SubscriberWorkflowPreferenceDtoTypedDict(TypedDict):
    enabled: bool
    r"""Whether notifications are enabled for this workflow"""
    channels: SubscriberPreferenceChannelsTypedDict
    r"""Channel-specific preference settings for this workflow"""
    overrides: List[SubscriberPreferenceOverrideDtoTypedDict]
    r"""List of preference overrides"""
    workflow: SubscriberPreferencesWorkflowInfoDtoTypedDict
    r"""Workflow information"""


class SubscriberWorkflowPreferenceDto(BaseModel):
    enabled: bool
    r"""Whether notifications are enabled for this workflow"""

    channels: SubscriberPreferenceChannels
    r"""Channel-specific preference settings for this workflow"""

    overrides: List[SubscriberPreferenceOverrideDto]
    r"""List of preference overrides"""

    workflow: SubscriberPreferencesWorkflowInfoDto
    r"""Workflow information"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ConfigurationsDtoTypedDict(TypedDict):
    inbound_webhook_enabled: NotRequired[bool]
    inbound_webhook_signing_key: NotRequired[str]


class ConfigurationsDto(BaseModel):
    inbound_webhook_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="inboundWebhookEnabled")
    ] = None

    inbound_webhook_signing_key: Annotated[
        Optional[str], pydantic.Field(alias="inboundWebhookSigningKey")
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .emailchanneloverrides import EmailChannelOverrides, EmailChannelOverridesTypedDict
from .severitylevelenum import SeverityLevelEnum
from .stepsoverrides import StepsOverrides, StepsOverridesTypedDict
from .subscriberpayloaddto import SubscriberPayloadDto, SubscriberPayloadDtoTypedDict
from .tenantpayloaddto import TenantPayloadDto, TenantPayloadDtoTypedDict
from .topicpayloaddto import TopicPayloadDto, TopicPayloadDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class ChannelsTypedDict(TypedDict):
    r"""Channel-specific overrides that apply to all steps of a particular channel type. Step-level overrides take precedence over channel-level overrides."""

    email: NotRequired[EmailChannelOverridesTypedDict]
    r"""Email channel specific overrides"""


class Channels(BaseModel):
    r"""Channel-specific overrides that apply to all steps of a particular channel type. Step-level overrides take precedence over channel-level overrides."""

    email: Optional[EmailChannelOverrides] = None
    r"""Email channel specific overrides"""


class OverridesTypedDict(TypedDict):
    r"""This could be used to override provider specific configurations"""

    steps: NotRequired[Dict[str, StepsOverridesTypedDict]]
    r"""This could be used to override provider specific configurations or layout at the step level"""
    channels: NotRequired[ChannelsTypedDict]
    r"""Channel-specific overrides that apply to all steps of a particular channel type. Step-level overrides take precedence over channel-level overrides."""
    providers: NotRequired[Dict[str, Dict[str, Any]]]
    r"""Overrides the provider configuration for the entire workflow and all steps"""
    email: NotRequired[Dict[str, Any]]
    r"""Override the email provider specific configurations for the entire workflow"""
    push: NotRequired[Dict[str, Any]]
    r"""Override the push provider specific configurations for the entire workflow"""
    sms: NotRequired[Dict[str, Any]]
    r"""Override the sms provider specific configurations for the entire workflow"""
    chat: NotRequired[Dict[str, Any]]
    r"""Override the chat provider specific configurations for the entire workflow"""
    layout_identifier: NotRequired[str]
    r"""Override the layout identifier for the entire workflow"""
    severity: NotRequired[SeverityLevelEnum]
    r"""Severity of the workflow"""


class Overrides(BaseModel):
    r"""This could be used to override provider specific configurations"""

    steps: Optional[Dict[str, StepsOverrides]] = None
    r"""This could be used to override provider specific configurations or layout at the step level"""

    channels: Optional[Channels] = None
    r"""Channel-specific overrides that apply to all steps of a particular channel type. Step-level overrides take precedence over channel-level overrides."""

    providers: Optional[Dict[str, Dict[str, Any]]] = None
    r"""Overrides the provider configuration for the entire workflow and all steps"""

    email: Annotated[
        Optional[Dict[str, Any]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""Override the email provider specific configurations for the entire workflow"""

    push: Annotated[
        Optional[Dict[str, Any]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""Override the push provider specific configurations for the entire workflow"""

    sms: Annotated[
        Optional[Dict[str, Any]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""Override the sms provider specific configurations for the entire workflow"""

    chat: Annotated[
        Optional[Dict[str, Any]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""Override the chat provider specific configurations for the entire workflow"""

    layout_identifier: Annotated[
        Optional[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="layoutIdentifier",
        ),
    ] = None
    r"""Override the layout identifier for the entire workflow"""

    severity: Optional[SeverityLevelEnum] = None
    r"""Severity of the workflow"""


To1TypedDict = TypeAliasType(
    "To1TypedDict", Union[TopicPayloadDtoTypedDict, SubscriberPayloadDtoTypedDict, str]
)


To1 = TypeAliasType("To1", Union[TopicPayloadDto, SubscriberPayloadDto, str])


ToTypedDict = TypeAliasType(
    "ToTypedDict",
    Union[
        TopicPayloadDtoTypedDict, SubscriberPayloadDtoTypedDict, List[To1TypedDict], str
    ],
)
r"""The recipients list of people who will receive the notification. Maximum number of recipients can be 100."""


To = TypeAliasType("To", Union[TopicPayloadDto, SubscriberPayloadDto, List[To1], str])
r"""The recipients list of people who will receive the notification. Maximum number of recipients can be 100."""


ActorTypedDict = TypeAliasType(
    "ActorTypedDict", Union[SubscriberPayloadDtoTypedDict, str]
)
r"""It is used to display the Avatar of the provided actor's subscriber id or actor object.

If a new actor object is provided, we will create a new subscriber in our system
"""


Actor = TypeAliasType("Actor", Union[SubscriberPayloadDto, str])
r"""It is used to display the Avatar of the provided actor's subscriber id or actor object.

If a new actor object is provided, we will create a new subscriber in our system
"""


TenantTypedDict = TypeAliasType(
    "TenantTypedDict", Union[TenantPayloadDtoTypedDict, str]
)
r"""It is used to specify a tenant context during trigger event.
Existing tenants will be updated with the provided details.
"""


Tenant = TypeAliasType("Tenant", Union[TenantPayloadDto, str])
r"""It is used to specify a tenant context during trigger event.
Existing tenants will be updated with the provided details.
"""


class TriggerEventRequestDtoContext2TypedDict(TypedDict):
    r"""Rich context object with id and optional data"""

    id: str
    data: NotRequired[Dict[str, Any]]
    r"""Optional additional context data"""


class TriggerEventRequestDtoContext2(BaseModel):
    r"""Rich context object with id and optional data"""

    id: str

    data: Optional[Dict[str, Any]] = None
    r"""Optional additional context data"""


TriggerEventRequestDtoContextTypedDict = TypeAliasType(
    "TriggerEventRequestDtoContextTypedDict",
    Union[TriggerEventRequestDtoContext2TypedDict, str],
)


TriggerEventRequestDtoContext = TypeAliasType(
    "TriggerEventRequestDtoContext", Union[TriggerEventRequestDtoContext2, str]
)


class TriggerEventRequestDtoTypedDict(TypedDict):
    workflow_id: str
    r"""The trigger identifier of the workflow you wish to send. This identifier can be found on the workflow page."""
    to: ToTypedDict
    r"""The recipients list of people who will receive the notification. Maximum number of recipients can be 100."""
    payload: NotRequired[Dict[str, Any]]
    r"""The payload object is used to pass additional custom information that could be
    used to render the workflow, or perform routing rules based on it.
    This data will also be available when fetching the notifications feed from the API to display certain parts of the UI.
    """
    overrides: NotRequired[OverridesTypedDict]
    r"""This could be used to override provider specific configurations"""
    transaction_id: NotRequired[str]
    r"""A unique identifier for deduplication. If the same **transactionId** is sent again,
    the trigger is ignored. Useful to prevent duplicate notifications. The retention period depends on your billing tier.
    """
    actor: NotRequired[ActorTypedDict]
    r"""It is used to display the Avatar of the provided actor's subscriber id or actor object.

    If a new actor object is provided, we will create a new subscriber in our system
    """
    tenant: NotRequired[TenantTypedDict]
    r"""It is used to specify a tenant context during trigger event.
    Existing tenants will be updated with the provided details.
    """
    context: NotRequired[Dict[str, TriggerEventRequestDtoContextTypedDict]]


class TriggerEventRequestDto(BaseModel):
    workflow_id: Annotated[str, pydantic.Field(alias="name")]
    r"""The trigger identifier of the workflow you wish to send. This identifier can be found on the workflow page."""

    to: To
    r"""The recipients list of people who will receive the notification. Maximum number of recipients can be 100."""

    payload: Optional[Dict[str, Any]] = None
    r"""The payload object is used to pass additional custom information that could be
    used to render the workflow, or perform routing rules based on it.
    This data will also be available when fetching the notifications feed from the API to display certain parts of the UI.
    """

    overrides: Optional[Overrides] = None
    r"""This could be used to override provider specific configurations"""

    transaction_id: Annotated[Optional[str], pydantic.Field(alias="transactionId")] = (
        None
    )
    r"""A unique identifier for deduplication. If the same **transactionId** is sent again,
    the trigger is ignored. Useful to prevent duplicate notifications. The retention period depends on your billing tier.
    """

    actor: Optional[Actor] = None
    r"""It is used to display the Avatar of the provided actor's subscriber id or actor object.

    If a new actor object is provided, we will create a new subscriber in our system
    """

    tenant: Optional[Tenant] = None
    r"""It is used to specify a tenant context during trigger event.
    Existing tenants will be updated with the provided details.
    """

    context: Optional[Dict[str, TriggerEventRequestDtoContext]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EmailControlDtoEditorType(str, Enum):
    r"""Type of editor to use for the body."""

    BLOCK = "block"
    HTML = "html"


class EmailControlDtoTypedDict(TypedDict):
    subject: str
    r"""Subject of the email."""
    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    body: NotRequired[str]
    r"""Body content of the email, either a valid Maily JSON object, or html string."""
    editor_type: NotRequired[EmailControlDtoEditorType]
    r"""Type of editor to use for the body."""
    disable_output_sanitization: NotRequired[bool]
    r"""Disable sanitization of the output."""
    layout_id: NotRequired[Nullable[str]]
    r"""Layout ID to use for the email. Null means no layout, undefined means default layout."""


class EmailControlDto(BaseModel):
    subject: str
    r"""Subject of the email."""

    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    body: Optional[str] = ""
    r"""Body content of the email, either a valid Maily JSON object, or html string."""

    editor_type: Annotated[
        Optional[EmailControlDtoEditorType], pydantic.Field(alias="editorType")
    ] = EmailControlDtoEditorType.BLOCK
    r"""Type of editor to use for the body."""

    disable_output_sanitization: Annotated[
        Optional[bool], pydantic.Field(alias="disableOutputSanitization")
    ] = False
    r"""Disable sanitization of the output."""

    layout_id: Annotated[OptionalNullable[str], pydantic.Field(alias="layoutId")] = (
        UNSET
    )
    r"""Layout ID to use for the email. Null means no layout, undefined means default layout."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "skip",
            "body",
            "editorType",
            "disableOutputSanitization",
            "layoutId",
        ]
        nullable_fields = ["layoutId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .getsubscriberpreferencesdto import (
    GetSubscriberPreferencesDto,
    GetSubscriberPreferencesDtoTypedDict,
)
from enum import Enum
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Criticality(str, Enum):
    CRITICAL = "critical"
    NON_CRITICAL = "nonCritical"
    ALL = "all"


class SubscribersControllerGetSubscriberPreferencesRequestTypedDict(TypedDict):
    subscriber_id: str
    criticality: NotRequired[Criticality]
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class SubscribersControllerGetSubscriberPreferencesRequest(BaseModel):
    subscriber_id: Annotated[
        str,
        pydantic.Field(alias="subscriberId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    criticality: Annotated[
        Optional[Criticality],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = Criticality.NON_CRITICAL

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class SubscribersControllerGetSubscriberPreferencesResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: GetSubscriberPreferencesDtoTypedDict


class SubscribersControllerGetSubscriberPreferencesResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: GetSubscriberPreferencesDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class MsTeamsChannelEndpointDtoTypedDict(TypedDict):
    team_id: str
    r"""MS Teams team ID"""
    channel_id: str
    r"""MS Teams channel ID"""


class MsTeamsChannelEndpointDto(BaseModel):
    team_id: Annotated[str, pydantic.Field(alias="teamId")]
    r"""MS Teams team ID"""

    channel_id: Annotated[str, pydantic.Field(alias="channelId")]
    r"""MS Teams channel ID"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .activitynotificationjobresponsedto import (
    ActivityNotificationJobResponseDto,
    ActivityNotificationJobResponseDtoTypedDict,
)
from .activitynotificationsubscriberresponsedto import (
    ActivityNotificationSubscriberResponseDto,
    ActivityNotificationSubscriberResponseDtoTypedDict,
)
from .activitynotificationtemplateresponsedto import (
    ActivityNotificationTemplateResponseDto,
    ActivityNotificationTemplateResponseDtoTypedDict,
)
from .activitytopicdto import ActivityTopicDto, ActivityTopicDtoTypedDict
from .severitylevelenum import SeverityLevelEnum
from .steptypeenum import StepTypeEnum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ActivityNotificationResponseDtoTypedDict(TypedDict):
    environment_id: str
    r"""Environment ID of the notification"""
    organization_id: str
    r"""Organization ID of the notification"""
    subscriber_id: str
    r"""Subscriber ID of the notification"""
    transaction_id: str
    r"""Transaction ID of the notification"""
    id: NotRequired[str]
    r"""Unique identifier of the notification"""
    template_id: NotRequired[str]
    r"""Template ID of the notification"""
    digested_notification_id: NotRequired[str]
    r"""Digested Notification ID"""
    created_at: NotRequired[str]
    r"""Creation time of the notification"""
    updated_at: NotRequired[str]
    r"""Last updated time of the notification"""
    channels: NotRequired[List[StepTypeEnum]]
    subscriber: NotRequired[ActivityNotificationSubscriberResponseDtoTypedDict]
    r"""Subscriber of the notification"""
    template: NotRequired[ActivityNotificationTemplateResponseDtoTypedDict]
    r"""Template of the notification"""
    jobs: NotRequired[List[ActivityNotificationJobResponseDtoTypedDict]]
    r"""Jobs of the notification"""
    payload: NotRequired[Dict[str, Any]]
    r"""Payload of the notification"""
    tags: NotRequired[List[str]]
    r"""Tags associated with the notification"""
    controls: NotRequired[Dict[str, Any]]
    r"""Controls associated with the notification"""
    to: NotRequired[Dict[str, Any]]
    r"""To field for subscriber definition"""
    topics: NotRequired[List[ActivityTopicDtoTypedDict]]
    r"""Topics of the notification"""
    severity: NotRequired[SeverityLevelEnum]
    r"""Severity of the workflow"""
    critical: NotRequired[bool]
    r"""Criticality of the notification"""
    context_keys: NotRequired[List[str]]
    r"""Context (single or multi) in which the notification was sent"""


class ActivityNotificationResponseDto(BaseModel):
    environment_id: Annotated[str, pydantic.Field(alias="_environmentId")]
    r"""Environment ID of the notification"""

    organization_id: Annotated[str, pydantic.Field(alias="_organizationId")]
    r"""Organization ID of the notification"""

    subscriber_id: Annotated[str, pydantic.Field(alias="_subscriberId")]
    r"""Subscriber ID of the notification"""

    transaction_id: Annotated[str, pydantic.Field(alias="transactionId")]
    r"""Transaction ID of the notification"""

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""Unique identifier of the notification"""

    template_id: Annotated[Optional[str], pydantic.Field(alias="_templateId")] = None
    r"""Template ID of the notification"""

    digested_notification_id: Annotated[
        Optional[str], pydantic.Field(alias="_digestedNotificationId")
    ] = None
    r"""Digested Notification ID"""

    created_at: Annotated[Optional[str], pydantic.Field(alias="createdAt")] = None
    r"""Creation time of the notification"""

    updated_at: Annotated[Optional[str], pydantic.Field(alias="updatedAt")] = None
    r"""Last updated time of the notification"""

    channels: Optional[List[StepTypeEnum]] = None

    subscriber: Optional[ActivityNotificationSubscriberResponseDto] = None
    r"""Subscriber of the notification"""

    template: Optional[ActivityNotificationTemplateResponseDto] = None
    r"""Template of the notification"""

    jobs: Optional[List[ActivityNotificationJobResponseDto]] = None
    r"""Jobs of the notification"""

    payload: Optional[Dict[str, Any]] = None
    r"""Payload of the notification"""

    tags: Optional[List[str]] = None
    r"""Tags associated with the notification"""

    controls: Optional[Dict[str, Any]] = None
    r"""Controls associated with the notification"""

    to: Optional[Dict[str, Any]] = None
    r"""To field for subscriber definition"""

    topics: Optional[List[ActivityTopicDto]] = None
    r"""Topics of the notification"""

    severity: Optional[SeverityLevelEnum] = None
    r"""Severity of the workflow"""

    critical: Optional[bool] = None
    r"""Criticality of the notification"""

    context_keys: Annotated[
        Optional[List[str]], pydantic.Field(alias="contextKeys")
    ] = None
    r"""Context (single or multi) in which the notification was sent"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateEnvironmentRequestDtoTypedDict(TypedDict):
    name: str
    r"""Name of the environment to be created"""
    color: str
    r"""Hex color code for the environment"""
    parent_id: NotRequired[str]
    r"""MongoDB ObjectId of the parent environment (optional)"""


class CreateEnvironmentRequestDto(BaseModel):
    name: str
    r"""Name of the environment to be created"""

    color: str
    r"""Hex color code for the environment"""

    parent_id: Annotated[Optional[str], pydantic.Field(alias="parentId")] = None
    r"""MongoDB ObjectId of the parent environment (optional)"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ReplyCallbackTypedDict(TypedDict):
    active: NotRequired[bool]
    r"""Indicates whether the reply callback is active."""
    url: NotRequired[str]
    r"""The URL to which replies should be sent."""


class ReplyCallback(BaseModel):
    active: Optional[bool] = None
    r"""Indicates whether the reply callback is active."""

    url: Optional[str] = None
    r"""The URL to which replies should be sent."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .authdto import AuthDto, AuthDtoTypedDict
from .workspacedto import WorkspaceDto, WorkspaceDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class TwoTypedDict(TypedDict):
    r"""Rich context object with id and optional data"""

    id: str
    data: NotRequired[Dict[str, Any]]
    r"""Optional additional context data"""


class Two(BaseModel):
    r"""Rich context object with id and optional data"""

    id: str

    data: Optional[Dict[str, Any]] = None
    r"""Optional additional context data"""


ContextTypedDict = TypeAliasType("ContextTypedDict", Union[TwoTypedDict, str])


Context = TypeAliasType("Context", Union[Two, str])


class CreateChannelConnectionRequestDtoTypedDict(TypedDict):
    integration_identifier: str
    r"""The identifier of the integration to use for this channel connection."""
    workspace: WorkspaceDtoTypedDict
    auth: AuthDtoTypedDict
    identifier: NotRequired[str]
    r"""The unique identifier for the channel connection. If not provided, one will be generated automatically."""
    subscriber_id: NotRequired[str]
    r"""The subscriber ID to link the channel connection to"""
    context: NotRequired[Dict[str, ContextTypedDict]]


class CreateChannelConnectionRequestDto(BaseModel):
    integration_identifier: Annotated[
        str, pydantic.Field(alias="integrationIdentifier")
    ]
    r"""The identifier of the integration to use for this channel connection."""

    workspace: WorkspaceDto

    auth: AuthDto

    identifier: Optional[str] = None
    r"""The unique identifier for the channel connection. If not provided, one will be generated automatically."""

    subscriber_id: Annotated[Optional[str], pydantic.Field(alias="subscriberId")] = None
    r"""The subscriber ID to link the channel connection to"""

    context: Optional[Dict[str, Context]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ExecutionDetailsSourceEnum(str, Enum):
    r"""Source of the execution detail"""

    CREDENTIALS = "Credentials"
    INTERNAL = "Internal"
    PAYLOAD = "Payload"
    WEBHOOK = "Webhook"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .resourceoriginenum import ResourceOriginEnum
from .stepissuesdto import StepIssuesDto, StepIssuesDtoTypedDict
from .steptypeenum import StepTypeEnum
from .throttlecontrolsmetadataresponsedto import (
    ThrottleControlsMetadataResponseDto,
    ThrottleControlsMetadataResponseDtoTypedDict,
)
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from pydantic import ConfigDict
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ThrottleStepResponseDtoType(str, Enum):
    r"""The type of throttle window."""

    FIXED = "fixed"
    DYNAMIC = "dynamic"


class ThrottleStepResponseDtoUnit(str, Enum):
    r"""The unit of time for the throttle window (required for fixed type)."""

    MINUTES = "minutes"
    HOURS = "hours"
    DAYS = "days"


class ThrottleStepResponseDtoControlValuesTypedDict(TypedDict):
    r"""Control values for the throttle step"""

    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    type: NotRequired[ThrottleStepResponseDtoType]
    r"""The type of throttle window."""
    amount: NotRequired[float]
    r"""The amount of time for the throttle window (required for fixed type)."""
    unit: NotRequired[ThrottleStepResponseDtoUnit]
    r"""The unit of time for the throttle window (required for fixed type)."""
    dynamic_key: NotRequired[str]
    r"""Key path to retrieve dynamic window value (required for dynamic type)."""
    threshold: NotRequired[float]
    r"""The maximum number of executions allowed within the window. Defaults to 1."""
    throttle_key: NotRequired[str]
    r"""Optional key for grouping throttle rules. If not provided, defaults to workflow and subscriber combination."""


class ThrottleStepResponseDtoControlValues(BaseModel):
    r"""Control values for the throttle step"""

    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, extra="allow"
    )
    __pydantic_extra__: Dict[str, Any] = pydantic.Field(init=False)

    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    type: Optional[ThrottleStepResponseDtoType] = ThrottleStepResponseDtoType.FIXED
    r"""The type of throttle window."""

    amount: Optional[float] = None
    r"""The amount of time for the throttle window (required for fixed type)."""

    unit: Optional[ThrottleStepResponseDtoUnit] = None
    r"""The unit of time for the throttle window (required for fixed type)."""

    dynamic_key: Annotated[Optional[str], pydantic.Field(alias="dynamicKey")] = None
    r"""Key path to retrieve dynamic window value (required for dynamic type)."""

    threshold: Optional[float] = 1
    r"""The maximum number of executions allowed within the window. Defaults to 1."""

    throttle_key: Annotated[Optional[str], pydantic.Field(alias="throttleKey")] = None
    r"""Optional key for grouping throttle rules. If not provided, defaults to workflow and subscriber combination."""

    @property
    def additional_properties(self):
        return self.__pydantic_extra__

    @additional_properties.setter
    def additional_properties(self, value):
        self.__pydantic_extra__ = value  # pyright: ignore[reportIncompatibleVariableOverride]


class ThrottleStepResponseDtoTypedDict(TypedDict):
    controls: ThrottleControlsMetadataResponseDtoTypedDict
    r"""Controls metadata for the throttle step"""
    variables: Dict[str, Any]
    r"""JSON Schema for variables, follows the JSON Schema standard"""
    step_id: str
    r"""Unique identifier of the step"""
    id: str
    r"""Database identifier of the step"""
    name: str
    r"""Name of the step"""
    slug: str
    r"""Slug of the step"""
    type: StepTypeEnum
    r"""Type of the step"""
    origin: ResourceOriginEnum
    r"""Origin of the layout"""
    workflow_id: str
    r"""Workflow identifier"""
    workflow_database_id: str
    r"""Workflow database identifier"""
    control_values: NotRequired[ThrottleStepResponseDtoControlValuesTypedDict]
    r"""Control values for the throttle step"""
    issues: NotRequired[StepIssuesDtoTypedDict]
    r"""Issues associated with the step"""


class ThrottleStepResponseDto(BaseModel):
    controls: ThrottleControlsMetadataResponseDto
    r"""Controls metadata for the throttle step"""

    variables: Dict[str, Any]
    r"""JSON Schema for variables, follows the JSON Schema standard"""

    step_id: Annotated[str, pydantic.Field(alias="stepId")]
    r"""Unique identifier of the step"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Database identifier of the step"""

    name: str
    r"""Name of the step"""

    slug: str
    r"""Slug of the step"""

    type: StepTypeEnum
    r"""Type of the step"""

    origin: ResourceOriginEnum
    r"""Origin of the layout"""

    workflow_id: Annotated[str, pydantic.Field(alias="workflowId")]
    r"""Workflow identifier"""

    workflow_database_id: Annotated[str, pydantic.Field(alias="workflowDatabaseId")]
    r"""Workflow database identifier"""

    control_values: Annotated[
        Optional[ThrottleStepResponseDtoControlValues],
        pydantic.Field(alias="controlValues"),
    ] = None
    r"""Control values for the throttle step"""

    issues: Optional[StepIssuesDto] = None
    r"""Issues associated with the step"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class IssueType(str, Enum):
    MISSING_VALUE = "MISSING_VALUE"
    MAX_LENGTH_ACCESSED = "MAX_LENGTH_ACCESSED"
    WORKFLOW_ID_ALREADY_EXISTS = "WORKFLOW_ID_ALREADY_EXISTS"
    DUPLICATED_VALUE = "DUPLICATED_VALUE"
    LIMIT_REACHED = "LIMIT_REACHED"


class RuntimeIssueDtoTypedDict(TypedDict):
    issue_type: IssueType
    message: str
    variable_name: NotRequired[str]


class RuntimeIssueDto(BaseModel):
    issue_type: Annotated[IssueType, pydantic.Field(alias="issueType")]

    message: str

    variable_name: Annotated[Optional[str], pydantic.Field(alias="variableName")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class LayoutResponseDtoSortField(str, Enum):
    CREATED_AT = "createdAt"
    UPDATED_AT = "updatedAt"
    NAME = "name"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channeltypeenum import ChannelTypeEnum
from .emailblock import EmailBlock, EmailBlockTypedDict
from .messagecta import MessageCTA, MessageCTATypedDict
from .messagestatusenum import MessageStatusEnum
from .subscriberresponsedto import SubscriberResponseDto, SubscriberResponseDtoTypedDict
from .workflowresponse import WorkflowResponse, WorkflowResponseTypedDict
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ContentTypedDict = TypeAliasType(
    "ContentTypedDict", Union[List[EmailBlockTypedDict], str]
)
r"""Content of the message, can be an email block or a string"""


Content = TypeAliasType("Content", Union[List[EmailBlock], str])
r"""Content of the message, can be an email block or a string"""


class MessageResponseDtoPayloadTypedDict(TypedDict):
    r"""The payload that was used to send the notification trigger"""


class MessageResponseDtoPayload(BaseModel):
    r"""The payload that was used to send the notification trigger"""


class MessageResponseDtoOverridesTypedDict(TypedDict):
    r"""Provider specific overrides used when triggering the notification"""


class MessageResponseDtoOverrides(BaseModel):
    r"""Provider specific overrides used when triggering the notification"""


class MessageResponseDtoTypedDict(TypedDict):
    environment_id: str
    r"""Environment ID where the message is sent"""
    organization_id: str
    r"""Organization ID associated with the message"""
    notification_id: str
    r"""Notification ID associated with the message"""
    subscriber_id: str
    r"""Subscriber ID associated with the message"""
    created_at: str
    r"""Creation date of the message"""
    transaction_id: str
    r"""Transaction ID associated with the message"""
    channel: ChannelTypeEnum
    r"""Channel type through which the message is sent"""
    read: bool
    r"""Indicates if the message has been read"""
    seen: bool
    r"""Indicates if the message has been seen"""
    cta: MessageCTATypedDict
    r"""Call to action associated with the message"""
    status: MessageStatusEnum
    r"""Status of the message"""
    id: NotRequired[str]
    r"""Unique identifier for the message"""
    template_id: NotRequired[Nullable[str]]
    r"""Template ID associated with the message"""
    message_template_id: NotRequired[Nullable[str]]
    r"""Message template ID"""
    subscriber: NotRequired[SubscriberResponseDtoTypedDict]
    r"""Subscriber details, if available"""
    template: NotRequired[WorkflowResponseTypedDict]
    r"""Workflow template associated with the message"""
    template_identifier: NotRequired[str]
    r"""Identifier for the message template"""
    delivered_at: NotRequired[List[str]]
    r"""Array of delivery dates for the message, if the message has multiple delivery dates, for example after being snoozed"""
    last_seen_date: NotRequired[str]
    r"""Last seen date of the message, if available"""
    last_read_date: NotRequired[str]
    r"""Last read date of the message, if available"""
    content: NotRequired[Nullable[ContentTypedDict]]
    r"""Content of the message, can be an email block or a string"""
    subject: NotRequired[str]
    r"""Subject of the message, if applicable"""
    snoozed_until: NotRequired[str]
    r"""Date when the message will be unsnoozed"""
    email: NotRequired[str]
    r"""Email address associated with the message, if applicable"""
    phone: NotRequired[str]
    r"""Phone number associated with the message, if applicable"""
    direct_webhook_url: NotRequired[str]
    r"""Direct webhook URL for the message, if applicable"""
    provider_id: NotRequired[str]
    r"""Provider ID associated with the message, if applicable"""
    device_tokens: NotRequired[List[str]]
    r"""Device tokens associated with the message, if applicable"""
    title: NotRequired[str]
    r"""Title of the message, if applicable"""
    feed_id: NotRequired[Nullable[str]]
    r"""Feed ID associated with the message, if applicable"""
    error_id: NotRequired[str]
    r"""Error ID if the message has an error"""
    error_text: NotRequired[str]
    r"""Error text if the message has an error"""
    payload: NotRequired[MessageResponseDtoPayloadTypedDict]
    r"""The payload that was used to send the notification trigger"""
    overrides: NotRequired[MessageResponseDtoOverridesTypedDict]
    r"""Provider specific overrides used when triggering the notification"""
    context_keys: NotRequired[List[str]]
    r"""Context (single or multi) in which the message was sent"""


class MessageResponseDto(BaseModel):
    environment_id: Annotated[str, pydantic.Field(alias="_environmentId")]
    r"""Environment ID where the message is sent"""

    organization_id: Annotated[str, pydantic.Field(alias="_organizationId")]
    r"""Organization ID associated with the message"""

    notification_id: Annotated[str, pydantic.Field(alias="_notificationId")]
    r"""Notification ID associated with the message"""

    subscriber_id: Annotated[str, pydantic.Field(alias="_subscriberId")]
    r"""Subscriber ID associated with the message"""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""Creation date of the message"""

    transaction_id: Annotated[str, pydantic.Field(alias="transactionId")]
    r"""Transaction ID associated with the message"""

    channel: ChannelTypeEnum
    r"""Channel type through which the message is sent"""

    read: bool
    r"""Indicates if the message has been read"""

    seen: bool
    r"""Indicates if the message has been seen"""

    cta: MessageCTA
    r"""Call to action associated with the message"""

    status: MessageStatusEnum
    r"""Status of the message"""

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""Unique identifier for the message"""

    template_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="_templateId")
    ] = UNSET
    r"""Template ID associated with the message"""

    message_template_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="_messageTemplateId")
    ] = UNSET
    r"""Message template ID"""

    subscriber: Optional[SubscriberResponseDto] = None
    r"""Subscriber details, if available"""

    template: Optional[WorkflowResponse] = None
    r"""Workflow template associated with the message"""

    template_identifier: Annotated[
        Optional[str], pydantic.Field(alias="templateIdentifier")
    ] = None
    r"""Identifier for the message template"""

    delivered_at: Annotated[
        Optional[List[str]], pydantic.Field(alias="deliveredAt")
    ] = None
    r"""Array of delivery dates for the message, if the message has multiple delivery dates, for example after being snoozed"""

    last_seen_date: Annotated[Optional[str], pydantic.Field(alias="lastSeenDate")] = (
        None
    )
    r"""Last seen date of the message, if available"""

    last_read_date: Annotated[Optional[str], pydantic.Field(alias="lastReadDate")] = (
        None
    )
    r"""Last read date of the message, if available"""

    content: OptionalNullable[Content] = UNSET
    r"""Content of the message, can be an email block or a string"""

    subject: Optional[str] = None
    r"""Subject of the message, if applicable"""

    snoozed_until: Annotated[Optional[str], pydantic.Field(alias="snoozedUntil")] = None
    r"""Date when the message will be unsnoozed"""

    email: Optional[str] = None
    r"""Email address associated with the message, if applicable"""

    phone: Optional[str] = None
    r"""Phone number associated with the message, if applicable"""

    direct_webhook_url: Annotated[
        Optional[str], pydantic.Field(alias="directWebhookUrl")
    ] = None
    r"""Direct webhook URL for the message, if applicable"""

    provider_id: Annotated[Optional[str], pydantic.Field(alias="providerId")] = None
    r"""Provider ID associated with the message, if applicable"""

    device_tokens: Annotated[
        Optional[List[str]], pydantic.Field(alias="deviceTokens")
    ] = None
    r"""Device tokens associated with the message, if applicable"""

    title: Optional[str] = None
    r"""Title of the message, if applicable"""

    feed_id: Annotated[OptionalNullable[str], pydantic.Field(alias="_feedId")] = UNSET
    r"""Feed ID associated with the message, if applicable"""

    error_id: Annotated[Optional[str], pydantic.Field(alias="errorId")] = None
    r"""Error ID if the message has an error"""

    error_text: Annotated[Optional[str], pydantic.Field(alias="errorText")] = None
    r"""Error text if the message has an error"""

    payload: Optional[MessageResponseDtoPayload] = None
    r"""The payload that was used to send the notification trigger"""

    overrides: Optional[MessageResponseDtoOverrides] = None
    r"""Provider specific overrides used when triggering the notification"""

    context_keys: Annotated[
        Optional[List[str]], pydantic.Field(alias="contextKeys")
    ] = None
    r"""Context (single or multi) in which the message was sent"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "_id",
            "_templateId",
            "_messageTemplateId",
            "subscriber",
            "template",
            "templateIdentifier",
            "deliveredAt",
            "lastSeenDate",
            "lastReadDate",
            "content",
            "subject",
            "snoozedUntil",
            "email",
            "phone",
            "directWebhookUrl",
            "providerId",
            "deviceTokens",
            "title",
            "_feedId",
            "errorId",
            "errorText",
            "payload",
            "overrides",
            "contextKeys",
        ]
        nullable_fields = ["_templateId", "_messageTemplateId", "content", "_feedId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import httpx
from typing import Optional
from dataclasses import dataclass

from novu_py.models import NovuError

MAX_MESSAGE_LEN = 10_000


@dataclass(unsafe_hash=True)
class APIError(NovuError):
    """The fallback error class if no more specific error class is matched."""

    def __init__(
        self, message: str, raw_response: httpx.Response, body: Optional[str] = None
    ):
        body_display = body or raw_response.text or '""'

        if message:
            message += ": "
        message += f"Status {raw_response.status_code}"

        headers = raw_response.headers
        content_type = headers.get("content-type", '""')
        if content_type != "application/json":
            if " " in content_type:
                content_type = f'"{content_type}"'
            message += f" Content-Type {content_type}"

        if len(body_display) > MAX_MESSAGE_LEN:
            truncated = body_display[:MAX_MESSAGE_LEN]
            remaining = len(body_display) - MAX_MESSAGE_LEN
            body_display = f"{truncated}...and {remaining} more chars"

        message += f". Body: {body_display}"
        message = message.strip()

        super().__init__(message, raw_response, body)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TriggerRecipientsTypeEnum(str, Enum):
    SUBSCRIBER = "Subscriber"
    TOPIC = "Topic"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .timerangedto import TimeRangeDto, TimeRangeDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class MondayTypedDict(TypedDict):
    r"""Monday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Monday(BaseModel):
    r"""Monday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""


class TuesdayTypedDict(TypedDict):
    r"""Tuesday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Tuesday(BaseModel):
    r"""Tuesday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""


class WednesdayTypedDict(TypedDict):
    r"""Wednesday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Wednesday(BaseModel):
    r"""Wednesday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""


class ThursdayTypedDict(TypedDict):
    r"""Thursday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Thursday(BaseModel):
    r"""Thursday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""


class FridayTypedDict(TypedDict):
    r"""Friday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Friday(BaseModel):
    r"""Friday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""


class SaturdayTypedDict(TypedDict):
    r"""Saturday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Saturday(BaseModel):
    r"""Saturday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""


class SundayTypedDict(TypedDict):
    r"""Sunday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Sunday(BaseModel):
    r"""Sunday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""


class WeeklyScheduleTypedDict(TypedDict):
    r"""Weekly schedule"""

    monday: NotRequired[MondayTypedDict]
    r"""Monday schedule"""
    tuesday: NotRequired[TuesdayTypedDict]
    r"""Tuesday schedule"""
    wednesday: NotRequired[WednesdayTypedDict]
    r"""Wednesday schedule"""
    thursday: NotRequired[ThursdayTypedDict]
    r"""Thursday schedule"""
    friday: NotRequired[FridayTypedDict]
    r"""Friday schedule"""
    saturday: NotRequired[SaturdayTypedDict]
    r"""Saturday schedule"""
    sunday: NotRequired[SundayTypedDict]
    r"""Sunday schedule"""


class WeeklySchedule(BaseModel):
    r"""Weekly schedule"""

    monday: Optional[Monday] = None
    r"""Monday schedule"""

    tuesday: Optional[Tuesday] = None
    r"""Tuesday schedule"""

    wednesday: Optional[Wednesday] = None
    r"""Wednesday schedule"""

    thursday: Optional[Thursday] = None
    r"""Thursday schedule"""

    friday: Optional[Friday] = None
    r"""Friday schedule"""

    saturday: Optional[Saturday] = None
    r"""Saturday schedule"""

    sunday: Optional[Sunday] = None
    r"""Sunday schedule"""


class ScheduleDtoTypedDict(TypedDict):
    is_enabled: bool
    r"""Schedule enabled"""
    weekly_schedule: NotRequired[WeeklyScheduleTypedDict]
    r"""Weekly schedule"""


class ScheduleDto(BaseModel):
    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Schedule enabled"""

    weekly_schedule: Annotated[
        Optional[WeeklySchedule], pydantic.Field(alias="weeklySchedule")
    ] = None
    r"""Weekly schedule"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .notificationtriggervariable import (
    NotificationTriggerVariable,
    NotificationTriggerVariableTypedDict,
)
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class NotificationTriggerDtoType(str, Enum):
    r"""Type of the trigger"""

    EVENT = "event"


class NotificationTriggerDtoTypedDict(TypedDict):
    type: NotificationTriggerDtoType
    r"""Type of the trigger"""
    identifier: str
    r"""Identifier of the trigger"""
    variables: List[NotificationTriggerVariableTypedDict]
    r"""Variables of the trigger"""
    subscriber_variables: NotRequired[List[NotificationTriggerVariableTypedDict]]
    r"""Subscriber variables of the trigger"""


class NotificationTriggerDto(BaseModel):
    type: NotificationTriggerDtoType
    r"""Type of the trigger"""

    identifier: str
    r"""Identifier of the trigger"""

    variables: List[NotificationTriggerVariable]
    r"""Variables of the trigger"""

    subscriber_variables: Annotated[
        Optional[List[NotificationTriggerVariable]],
        pydantic.Field(alias="subscriberVariables"),
    ] = None
    r"""Subscriber variables of the trigger"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .apikeydto import APIKeyDto, APIKeyDtoTypedDict
from enum import Enum
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EnvironmentResponseDtoType(str, Enum):
    r"""Type of the environment"""

    DEV = "dev"
    PROD = "prod"


class EnvironmentResponseDtoTypedDict(TypedDict):
    id: str
    r"""Unique identifier of the environment"""
    name: str
    r"""Name of the environment"""
    organization_id: str
    r"""Organization ID associated with the environment"""
    identifier: str
    r"""Unique identifier for the environment"""
    type: NotRequired[Nullable[EnvironmentResponseDtoType]]
    r"""Type of the environment"""
    api_keys: NotRequired[List[APIKeyDtoTypedDict]]
    r"""List of API keys associated with the environment"""
    parent_id: NotRequired[str]
    r"""Parent environment ID"""
    slug: NotRequired[str]
    r"""URL-friendly slug for the environment"""


class EnvironmentResponseDto(BaseModel):
    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Unique identifier of the environment"""

    name: str
    r"""Name of the environment"""

    organization_id: Annotated[str, pydantic.Field(alias="_organizationId")]
    r"""Organization ID associated with the environment"""

    identifier: str
    r"""Unique identifier for the environment"""

    type: OptionalNullable[EnvironmentResponseDtoType] = UNSET
    r"""Type of the environment"""

    api_keys: Annotated[Optional[List[APIKeyDto]], pydantic.Field(alias="apiKeys")] = (
        None
    )
    r"""List of API keys associated with the environment"""

    parent_id: Annotated[Optional[str], pydantic.Field(alias="_parentId")] = None
    r"""Parent environment ID"""

    slug: Optional[str] = None
    r"""URL-friendly slug for the environment"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "apiKeys", "_parentId", "slug"]
        nullable_fields = ["type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TopicSubscriberIdentifierDtoTypedDict(TypedDict):
    identifier: str
    r"""Unique identifier for this subscription"""
    subscriber_id: str
    r"""The subscriber ID"""
    name: NotRequired[str]
    r"""The name of the subscription"""


class TopicSubscriberIdentifierDto(BaseModel):
    identifier: str
    r"""Unique identifier for this subscription"""

    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""The subscriber ID"""

    name: Optional[str] = None
    r"""The name of the subscription"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class UpdatedSubscriberDtoTypedDict(TypedDict):
    subscriber_id: str
    r"""The ID of the subscriber that was updated."""


class UpdatedSubscriberDto(BaseModel):
    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""The ID of the subscriber that was updated."""
